import unittest

from vha_toolbox import seconds_to_humantime


class TestSecondsToHumanTime(unittest.TestCase):
    def test_seconds_to_humantime_1(self):
        self.assertEqual(seconds_to_humantime(3660), '1 hour and 1 minute')

    def test_seconds_to_humantime_2(self):
        self.assertEqual(seconds_to_humantime(3660, include_seconds=False), '1 hour and 1 minute')

    def test_seconds_to_humantime_3(self):
        self.assertEqual(seconds_to_humantime(60), '1 minute')

    def test_seconds_to_humantime_4(self):
        self.assertEqual(seconds_to_humantime(120), '2 minutes')

    def test_seconds_to_humantime_5(self):
        self.assertEqual(seconds_to_humantime(0), '0 seconds')

    def test_seconds_to_humantime_6(self):
        self.assertEqual(seconds_to_humantime(31536000 + 2592000 + 86400 + 3600 + 60 + 1), '1 year, 1 month, 1 day, 1 hour, 1 minute and 1 second')

    def test_seconds_to_humantime_7(self):
        self.assertEqual(seconds_to_humantime(61, include_seconds=False), '1 minute')

    def test_seconds_to_humantime_8(self):
        self.assertEqual(seconds_to_humantime(6646.15384615), '1 hour, 50 minutes and 46 seconds')


if __name__ == '__main__':
    unittest.main()



import unittest

from vha_toolbox import replace_multiple_substrings


class ReplaceMultipleSubstringsTestCase(unittest.TestCase):
    def test_replace_multiple_substrings(self):
        replacements = {
            'apple': 'orange',
            'banana': 'grape',
            'cherry': 'melon'
        }
        original_string = 'I have an apple, a banana, and a cherry.'
        expected_result = 'I have an orange, a grape, and a melon.'

        result = replace_multiple_substrings(original_string, replacements)
        self.assertEqual(result, expected_result)

    def test_replace_multiple_occurrences(self):
        replacements = {
            'apple': 'orange',
            'banana': 'grape',
            'cherry': 'melon'
        }
        original_string = 'I have an apple, a banana, and a cherry. An apple is sweet, but a cherry is sour.'
        expected_result = 'I have an orange, a grape, and a melon. An orange is sweet, but a melon is sour.'
        result = replace_multiple_substrings(original_string, replacements)
        self.assertEqual(result, expected_result)

    def test_replace_empty_string(self):
        replacements = {
            'apple': 'orange',
            'banana': 'grape',
            'cherry': 'melon'
        }
        original_string = ''
        expected_result = ''
        result = replace_multiple_substrings(original_string, replacements)
        self.assertEqual(result, expected_result)

    def test_replace_empty_replacements_dict(self):
        replacements = {}
        original_string = 'I have an apple, a banana, and a cherry.'
        expected_result = 'I have an apple, a banana, and a cherry.'
        result = replace_multiple_substrings(original_string, replacements)
        self.assertEqual(result, expected_result)

    def test_replace_overlapping_substrings(self):
        replacements = {
            'apple': 'orange',
            'orange': 'grapefruit'
        }
        original_string = 'I have an apple and an orange.'
        expected_result = 'I have an orange and an grapefruit.'
        result = replace_multiple_substrings(original_string, replacements)
        self.assertEqual(result, expected_result)


if __name__ == '__main__':
    unittest.main()



import unittest

from vha_toolbox import format_readable_size, to_bytes, sort_human_readable_sizes


class FormatSizeTestCase(unittest.TestCase):
    def test_format_readable_size(self):
        self.assertEqual(format_readable_size(0), "0.0 B")
        self.assertEqual(format_readable_size(1023), "1023.0 B")
        self.assertEqual(format_readable_size(1024), "1.0 KB")
        self.assertEqual(format_readable_size(123456789), "117.7 MB")
        self.assertEqual(format_readable_size(1000000000000), "931.3 GB")
        self.assertEqual(format_readable_size(999999999999999999), "888.2 PB")

    def test_format_readable_size_with_different_decimal_place(self):
        self.assertEqual(format_readable_size(1023, decimal_places=0), "1023 B")
        self.assertEqual(format_readable_size(1023, decimal_places=2), "1023.00 B")
        self.assertEqual(format_readable_size(1024, decimal_places=0), "1 KB")
        self.assertEqual(format_readable_size(123456789, decimal_places=1), "117.7 MB")
        self.assertEqual(format_readable_size(123456789, decimal_places=2), "117.74 MB")
        self.assertEqual(format_readable_size(1000000000000, decimal_places=3), "931.323 GB")
        self.assertEqual(format_readable_size(999999999999999999, decimal_places=4), "888.1784 PB")

    def test_format_readable_size_error(self):
        self.assertRaises(ValueError, format_readable_size, -100)
        self.assertRaises(ValueError, format_readable_size, -1)

    def test_to_bytes(self):
        self.assertEqual(to_bytes("0 B"), 0)
        self.assertEqual(to_bytes("1023 B"), 1023)
        self.assertEqual(to_bytes("1 KB"), 1024)
        self.assertEqual(to_bytes("117.73 MB"), 123448852)
        self.assertEqual(to_bytes("117.7 MB"), 123417395)
        self.assertEqual(to_bytes("931.3 GB"), 999975760691)
        self.assertEqual(to_bytes("888.1784 PB"), 999999977819630848)

    def test_to_bytes_wrong_unit(self):
        self.assertEqual(to_bytes("10230 B"), 10230)
        self.assertEqual(to_bytes("0.001 MB"), 1048)

    def test_to_bytes_error(self):
        self.assertRaises(ValueError, to_bytes, "")
        self.assertRaises(ValueError, to_bytes, "0")
        self.assertRaises(ValueError, to_bytes, "1")
        self.assertRaises(ValueError, to_bytes, "1.0 B B")
        self.assertRaises(ValueError, to_bytes, "-1")
        self.assertRaises(ValueError, to_bytes, "-1.0 B")

    def test_sort_human_readable_sizes(self):
        sizes = ["1 KB", "117.7 MB", "1023 B", "931.3 GB", "888.2 PB"]
        expected = ["1023 B", "1 KB", "117.7 MB", "931.3 GB", "888.2 PB"]
        self.assertEqual(sort_human_readable_sizes(sizes), expected)

        sizes = ["1 KB", "117.7 MB", "1023 B", "931.3 GB", "888.2 PB", "0 B"]
        expected = ["0 B", "1023 B", "1 KB", "117.7 MB", "931.3 GB", "888.2 PB"]
        self.assertEqual(sort_human_readable_sizes(sizes), expected)

        sizes = ["1 KB", "117.7 MB", "1023 B", "931.3 GB", "888.2 PB", "0 B", "1 KB"]
        expected = ["0 B", "1023 B", "1 KB", "1 KB", "117.7 MB", "931.3 GB", "888.2 PB"]
        self.assertEqual(sort_human_readable_sizes(sizes), expected)

        sizes = ["1.1 KB", "117.7 MB", "1023 B", "931.3 GB", "888.2 PB", "0 B", "1 KB", "1 KB"]
        expected = ["0 B", "1023 B", "1 KB", "1 KB", "1.1 KB", "117.7 MB", "931.3 GB", "888.2 PB"]
        self.assertEqual(sort_human_readable_sizes(sizes), expected)

    def test_sort_human_readable_sizes_reverse(self):
        sizes = ["1 KB", "117.7 MB", "1023 B", "931.3 GB", "888.2 PB"]
        expected = ["888.2 PB", "931.3 GB", "117.7 MB", "1 KB", "1023 B"]
        self.assertEqual(sort_human_readable_sizes(sizes, reverse=True), expected)

        sizes = ["1 KB", "117.7 MB", "1023 B", "931.3 GB", "888.2 PB", "0 B"]
        expected = ["888.2 PB", "931.3 GB", "117.7 MB", "1 KB", "1023 B", "0 B"]
        self.assertEqual(sort_human_readable_sizes(sizes, reverse=True), expected)

        sizes = ["1 KB", "117.7 MB", "1023 B", "931.3 GB", "888.2 PB", "0 B", "1 KB"]
        expected = ["888.2 PB", "931.3 GB", "117.7 MB", "1 KB", "1 KB", "1023 B", "0 B"]
        self.assertEqual(sort_human_readable_sizes(sizes, reverse=True), expected)

        sizes = ["1.1 KB", "117.7 MB", "1023 B", "931.3 GB", "888.2 PB", "0 B", "1 KB", "1 KB"]
        expected = ["888.2 PB", "931.3 GB", "117.7 MB", "1.1 KB", "1 KB", "1 KB", "1023 B", "0 B"]
        self.assertEqual(sort_human_readable_sizes(sizes, reverse=True), expected)

    def test_sort_human_readable_sizes_error(self):
        self.assertRaises(ValueError, sort_human_readable_sizes, ["1 KB", "117.7 Hello"])
        self.assertRaises(ValueError, sort_human_readable_sizes, ["1 KB", "117.7 MB", ""])
        self.assertRaises(ValueError, sort_human_readable_sizes, ["-1 KB"])


if __name__ == '__main__':
    unittest.main()



import unittest

from vha_toolbox import sql_helper


class SQLHelperTestCase(unittest.TestCase):
    def test_get_sql_1(self):
        table_name = 'my_table'
        columns = ['column_1', 'column_2']
        sql = sql_helper.SQL(table_name, columns)
        result = str(sql)
        expected_sql = 'SELECT column_1, column_2 FROM my_table'
        self.assertEqual(result, expected_sql)

    def test_get_sql_2(self):
        table_name = 'my_table'
        columns = ['column_1', 'column_2']
        sql = sql_helper.SQL(table_name, columns)
        sql.add_with('with_1', 'SELECT * FROM other_table')
        sql.add_with('with_2', 'SELECT * FROM other_table')
        result = str(sql)
        expected_sql = 'WITH with_1 AS (SELECT * FROM other_table)\nWITH with_2 AS (SELECT * FROM other_table)\nSELECT column_1, column_2 FROM my_table'
        self.assertEqual(result, expected_sql)

    def test_get_sql_3(self):
        table_name = 'my_table'
        columns = ['column_1', 'column_2']
        sql = sql_helper.SQL(table_name, columns)
        sql.add_with('with_1', 'SELECT * FROM other_table')
        sql.add_with('with_2', 'SELECT * FROM other_table')
        sql.set_where('column_1 > 5')
        result = str(sql)
        expected_sql = 'WITH with_1 AS (SELECT * FROM other_table)\nWITH with_2 AS (SELECT * FROM other_table)\nSELECT column_1, column_2 FROM my_table WHERE column_1 > 5'
        self.assertEqual(result, expected_sql)

    def test_get_sql_4(self):
        table_name = 'my_table'
        columns = ['column_1', 'column_2']
        sql = sql_helper.SQL(table_name, columns)
        sql.add_with('with_1', 'SELECT * FROM other_table')
        sql.add_with('with_2', 'SELECT * FROM other_table')
        sql.set_where('column_1 > 5')
        sql.set_limit(10)
        result = str(sql)
        expected_sql = 'WITH with_1 AS (SELECT * FROM other_table)\nWITH with_2 AS (SELECT * FROM other_table)\nSELECT column_1, column_2 FROM my_table WHERE column_1 > 5 LIMIT 10'
        self.assertEqual(result, expected_sql)

    def test_get_sql_5(self):
        table_name = 'my_table'
        columns = ['column_1', 'column_2']
        sql = sql_helper.SQL(table_name, columns)
        sql.add_with('with_1', 'SELECT * FROM other_table')
        sql.add_with('with_2', 'SELECT * FROM other_table')
        sql.set_where('column_1 > 5')
        sql.set_limit(10)
        sql.add_join('JOIN other_table ON my_table.id = other_table.id')
        result = str(sql)
        expected_sql = 'WITH with_1 AS (SELECT * FROM other_table)\nWITH with_2 AS (SELECT * FROM other_table)\nSELECT column_1, column_2 FROM my_table JOIN other_table ON my_table.id = other_table.id WHERE column_1 > 5 LIMIT 10'
        self.assertEqual(result, expected_sql)

    def test_get_sql_6(self):
        table_name = 'my_table'
        columns = ['column_1', 'column_2']
        sql = sql_helper.SQL(table_name, columns)
        sql.add_with('with_1', 'SELECT * FROM other_table')
        sql.add_with('with_2', 'SELECT * FROM other_table')
        sql.set_where('column_1 > 5')
        sql.set_limit(10)
        sql.add_join('JOIN other_table ON my_table.id = other_table.id')
        sql.add_join('JOIN other_table ON my_table.id = other_table.id')
        sql.remove_with('with_1')
        result = str(sql)
        expected_sql = 'WITH with_2 AS (SELECT * FROM other_table)\nSELECT column_1, column_2 FROM my_table JOIN other_table ON my_table.id = other_table.id JOIN other_table ON my_table.id = other_table.id WHERE column_1 > 5 LIMIT 10'
        self.assertEqual(result, expected_sql)

    def test_create_sql_used_in_with(self):
        table_name_1 = 'my_with_table'
        columns_1 = ['column_with_1', 'column_with_2']
        sql_1 = sql_helper.SQL(table_name_1, columns_1)
        sql_1_result = str(sql_1)
        expected_sql_1 = 'SELECT column_with_1, column_with_2 FROM my_with_table'
        self.assertEqual(sql_1_result, expected_sql_1)
        table_name = 'my_table'
        columns = ['column_1', 'column_2']
        sql = sql_helper.SQL(table_name, columns)
        sql.add_with('with_1', sql_1)
        sql.add_with('with_2', 'SELECT * FROM other_table')
        sql.set_where('column_1 > 5')
        sql.set_limit(10)
        sql.add_join('JOIN other_table ON my_table.id = other_table.id')
        result = str(sql)
        expected_sql = 'WITH with_1 AS (SELECT column_with_1, column_with_2 FROM my_with_table)\nWITH with_2 AS (SELECT * FROM other_table)\nSELECT column_1, column_2 FROM my_table JOIN other_table ON my_table.id = other_table.id WHERE column_1 > 5 LIMIT 10'
        self.assertEqual(result, expected_sql)


if __name__ == '__main__':
    unittest.main()



import unittest
from PIL import Image
from vha_toolbox import should_rotate_by_ratio


class ShouldRotateByRatioTestCase(unittest.TestCase):

    def _img(self, w, h):
        """Utility pour créer une image PIL en mémoire."""
        return Image.new("RGB", (w, h))

    def test_no_rotation_on_square_frame(self):
        frame = self._img(1000, 1000)
        img = self._img(3000, 2000)

        result = should_rotate_by_ratio(
            img, frame,
            frame_target_ratio=2/3,
            img_target_ratio=3/2
        )
        self.assertFalse(result)

    def test_rotation_for_matching_ratios_and_orientation_flip(self):
        frame = self._img(2000, 3000)  # portrait ≈ 0.66
        img = self._img(3000, 2000)    # landscape ≈ 1.5

        result = should_rotate_by_ratio(
            img, frame,
            frame_target_ratio=2/3,
            img_target_ratio=3/2
        )
        self.assertTrue(result)

    def test_no_rotation_if_ratios_do_not_match(self):
        frame = self._img(2000, 3000)  # ratio ≈ 0.66
        img = self._img(4000, 2000)    # ratio = 2.0 > mauvais ratio

        result = should_rotate_by_ratio(
            img, frame,
            frame_target_ratio=2/3,
            img_target_ratio=3/2
        )
        self.assertFalse(result)

    def test_no_rotation_if_orientation_not_flipped(self):
        frame = self._img(2000, 3000)  # portrait ratio 2/3
        img = self._img(2000, 3000)    # portrait ratio 2/3

        result = should_rotate_by_ratio(
            img, frame,
            frame_target_ratio=2/3,
            img_target_ratio=3/2
        )
        self.assertFalse(result)

    def test_rotation_without_orientation_requirement(self):
        frame = self._img(2000, 3000)
        img = self._img(3000, 2000)

        result = should_rotate_by_ratio(
            img, frame,
            frame_target_ratio=2/3,
            img_target_ratio=3/2,
            require_orientation_flip=False,
        )
        self.assertTrue(result)


if __name__ == '__main__':
    unittest.main()



import hashlib
import json
from pathlib import Path
from typing import Any, Iterable, Optional, Union


def compute_file_hash(
    file_path: Union[str, Path],
    extra_paths: Optional[Iterable[Union[str, Path]]] = None
) -> str:
    """
    Compute a stable SHA-256 hash for a file, optionally combined with other files.
    Only the first 8 hexadecimal characters of the hash are returned.

    Args:
        file_path (str | Path): The path to the primary file.
        extra_paths (list[str | Path] | None, optional): Additional file paths whose
            contents should contribute to the hash. Defaults to None.

    Returns:
        str: The first 8 characters of the resulting SHA-256 hash.

    Example:
        >>> compute_file_hash("image.jpg")
        'a94f2c1b'
        >>> compute_file_hash("image.jpg", ["frame.png"])
        'e12d99fa'

    Raises:
        FileNotFoundError: If the primary file does not exist.
        ValueError: If a provided path is invalid.
    """

    def _update_hasher(p: Path, hasher_obj: hashlib._hashlib.HASH):
        """Internal helper to update the hash with the content of a file."""
        if not p.exists() or not p.is_file():
            raise FileNotFoundError(f"Path '{p}' not found or not a file.")

        with p.open("rb") as f:
            while chunk := f.read(8192):
                hasher_obj.update(chunk)

    # Normalize main path
    main_path = Path(file_path)

    if not main_path.exists():
        raise FileNotFoundError(f"Path '{main_path}' not found.")

    hasher = hashlib.sha256()

    # Hash the primary file
    _update_hasher(main_path, hasher)

    # Hash extra files if provided
    if extra_paths:
        for ep in extra_paths:
            if ep is None:
                continue
            path = Path(ep)
            _update_hasher(path, hasher)

    # Return short hash
    return hasher.hexdigest()[:8]


def compute_stable_hash(
    values: Any,
    prefix: Optional[str] = None,
    length: int = 8
) -> str:
    """
    Compute a stable SHA-256 hash from any list of Python values.

    - Supporte : str, int, float, bool, None
    - Supporte : dicts (triés pour stabilité)
    - Supporte : listes, tuples, sets
    - Supporte : Path
    - Supporte : bytes / bytearray
    - Supporte : objets arbitraires en fallback repr()

    Args:
        values (Any): Single value or iterable of values.
        prefix (str | None): Optionnel, ajouté avant le hash final (ex.: "img_", "row_").
        length (int): Nombre de caractères du hash retourné (par défaut : 8).

    Returns:
        str: Short stable SHA-256 hash.

    Example:
        >>> compute_stable_hash(["image1.png", 3000, {"x": 12}])
        '9f3ac1d2'

        >>> compute_stable_hash("hello", prefix="p_")
        'p_a1b2c3d4'
    """

    def normalize(v):
        """Normalize a Python value into a deterministic JSON-serializable value."""
        if v is None:
            return None

        # Path → string
        if isinstance(v, Path):
            return str(v)

        # bytes → hex string
        if isinstance(v, (bytes, bytearray)):
            return v.hex()

        # dict → trié pour stabilité
        if isinstance(v, dict):
            return {k: normalize(v[k]) for k in sorted(v)}

        # list / tuple / set → liste triée si set
        if isinstance(v, (list, tuple)):
            return [normalize(x) for x in v]

        if isinstance(v, set):
            return sorted(normalize(x) for x in v)

        # primitive types
        if isinstance(v, (int, float, str, bool)):
            return v

        # fallback pour objets arbitraires
        return repr(v)

    # Si values n'est pas iterable → l’envelopper dans une liste
    if not isinstance(values, (list, tuple, set)):
        values = [values]

    # Normalisation profonde
    norm = normalize(values)

    # Conversion en JSON stable
    encoded = json.dumps(norm, sort_keys=True, separators=(",", ":")).encode("utf-8")

    # Hash
    digest = hashlib.sha256(encoded).hexdigest()[:length]

    return f"{prefix}{digest}" if prefix else digest



import unittest

from vha_toolbox import highlight_text


class HighlightTextTestCase(unittest.TestCase):
    def test_highlight_text(self):
        text = "This is a sample text to test the highlighting function."
        words = ["sample", "function"]
        expected_output = "This is a <span>sample</span> text to test the highlighting <span>function</span>."
        result = highlight_text(text, words)
        self.assertEqual(result, expected_output)

    def test_highlight_text_custom_tags(self):
        text = "This is a sample text to test the highlighting function."
        words = ["sample", "function"]
        expected_output = "This is a [sample] text to test the highlighting [function]."

        result = highlight_text(text, words, start_tag='[', end_tag=']')
        self.assertEqual(result, expected_output)

    def test_highlight_text_custom_tags_contained_in_text(self):
        text = "This is a sample text to test the highlighting [function]."
        words = ["sample", "function"]
        expected_output = "This is a [sample] text to test the highlighting [[function]]."

        result = highlight_text(text, words, start_tag='[', end_tag=']')
        self.assertEqual(result, expected_output)

    def test_highlight_text_custom_tags_contained_in_text_2(self):
        text = "This is a sample text to test the highlighting [function]."
        words = ["sample", "function"]
        expected_output = "This is a [[sample]] text to test the highlighting [[[function]]]."

        result = highlight_text(text, words, start_tag='[[', end_tag=']]')
        self.assertEqual(result, expected_output)

    def test_highlight_text_case_insensitive(self):
        text = "This is a case-insensitive Test."
        words = ["TEST"]
        expected_output = "This is a case-insensitive <span>Test</span>."
        result = highlight_text(text, words, case_insensitive=True)
        self.assertEqual(result, expected_output)

    def test_highlight_text_with_substring_overlapping(self):
        text = "This is a sample text."
        words = ["sam", "ample"]
        expected_output = "This is a <span>sample</span> text."
        result = highlight_text(text, words)
        self.assertEqual(result, expected_output)

    def test_highlight_text_with_overlapping(self):
        text = "This is a test with overlapping words."
        words = ["with overlapping", "overlapping words"]
        expected_output = "This is a test <span>with overlapping words</span>."
        result = highlight_text(text, words)
        self.assertEqual(result, expected_output)

    def test_highlight_text_with_boundaries(self):
        text = "words and word"
        words = ["word"]
        expected_output = "words and <span>word</span>"
        result = highlight_text(text, words, word_boundaries=True)
        self.assertEqual(result, expected_output)

    def test_highlight_text_with_multiple_overlapping(self):
        text = "This is a test with overlapping words."
        words = ["test with", "with overlapping", "overlapping words"]
        expected_output = "This is a <span>test with overlapping words</span>."
        result = highlight_text(text, words)
        self.assertEqual(result, expected_output)

    def test_highlight_text_no_words_to_highlight(self):
        text = "This is a test with no words to highlight."
        words = []
        expected_output = "This is a test with no words to highlight."
        result = highlight_text(text, words)
        self.assertEqual(result, expected_output)

    def test_highlight_text_empty_text(self):
        text = ""
        words = ["highlight", "function"]
        expected_output = ""
        result = highlight_text(text, words)
        self.assertEqual(result, expected_output)

    def test_highlight_text_multiple_occurrences(self):
        text = "This is a test with multiple occurrences of the word 'test'."
        words = ["test"]
        expected_output = "This is a <span>test</span> with multiple occurrences of the word '<span>test</span>'."
        result = highlight_text(text, words)
        self.assertEqual(result, expected_output)

    def test_highlight_words_in_long_text_multiple_occurrences(self):
        text = "This is a long text to test the highlighting function. This text is long and repetitive. This is a long text to test the highlighting function. This text is long and repetitive."
        words = ["long", "text", "repetitive", "highlighting", "function"]
        expected_output = "This is a <span>long</span> <span>text</span> to test the <span>highlighting</span> <span>function</span>. This <span>text</span> is <span>long</span> and <span>repetitive</span>. This is a <span>long</span> <span>text</span> to test the <span>highlighting</span> <span>function</span>. This <span>text</span> is <span>long</span> and <span>repetitive</span>."
        result = highlight_text(text, words)
        self.assertEqual(result, expected_output)

    def test_highlight_text_no_accents_insensitive(self):
        text = "Café nötë Hello wörld!"
        words = ["cafe", "note", "world"]
        expected_output = "Café nötë Hello wörld!"

        result = highlight_text(text, words)
        self.assertEqual(result, expected_output)

    def test_highlight_text_accents_insensitive(self):
        text = "Café nötë Hello wörld!"
        words = ["cafe", "note", "world"]
        expected_output = "Café <span>nötë</span> Hello <span>wörld</span>!"

        result = highlight_text(text, words, accents_insensitive=True)
        self.assertEqual(result, expected_output)

    def test_highlight_text_case_and_accents_insensitive(self):
        text = "Café nötë Hello wörld!"
        words = ["cafe", "note", "world"]
        expected_output = "<span>Café</span> <span>nötë</span> Hello <span>wörld</span>!"

        result = highlight_text(text, words, case_insensitive=True, accents_insensitive=True)
        self.assertEqual(result, expected_output)


if __name__ == '__main__':
    unittest.main()



import unittest

from vha_toolbox import text_to_html


class TextToHtmlTestCase(unittest.TestCase):
    def test_text_to_html_1(self):
        text = 'Hello world!'
        result = text_to_html(text)
        self.assertEqual(result, '<p>Hello world!</p>')

    def test_text_to_html_2(self):
        text = 'Hello\nworld!'
        result = text_to_html(text)
        self.assertEqual(result, '<p>Hello</p><p>world!</p>')

    def test_text_to_html_3(self):
        text = 'Hello\nworld!'
        replacements = ['\n']
        result = text_to_html(text, replacements)
        self.assertEqual(result, '<p>Hello</p><p>world!</p>')

    def test_text_to_html_4(self):
        text = 'Hello\nworld!'
        replacements = []
        result = text_to_html(text, replacements)
        self.assertEqual(result, '<p>Hello</p><p>world!</p>')

    def test_text_to_html_5(self):
        text = 'Hello\nworld!'
        replacements = ['\n', 'world']
        result = text_to_html(text, replacements)
        self.assertEqual(result, '<p>Hello</p><p></p><p>!</p>')


if __name__ == '__main__':
    unittest.main()



import unittest

from vha_toolbox import truncate_with_ellipsis


class TruncateWithEllipsisTestCase(unittest.TestCase):
    def test_truncate_with_ellipsis(self):
        self.assertEqual(truncate_with_ellipsis('Hello world!', 5), 'Hello...')
        self.assertEqual(truncate_with_ellipsis('Hello world!', 7), 'Hello w...')
        self.assertEqual(truncate_with_ellipsis('Hello world!', 11), 'Hello world...')
        self.assertEqual(truncate_with_ellipsis('Hello world!', 12), 'Hello world!')

    def test_truncate_with_ellipsis_with_blank(self):
        self.assertEqual(truncate_with_ellipsis('Hello world! ', 6), 'Hello...')
        self.assertEqual(truncate_with_ellipsis('Hello world! ', 13), 'Hello world! ')

    def test_truncate_with_ellipsis_with_blank_disabled(self):
        self.assertEqual(truncate_with_ellipsis('Hello world! ', 6, del_blank=False), 'Hello ...')
        self.assertEqual(truncate_with_ellipsis('Hello world! ', 13, del_blank=False), 'Hello world! ')

    def test_truncate_with_ellipsis_with_ellipsis(self):
        self.assertEqual(truncate_with_ellipsis('Hello world!', 5, ellipsis='...!'), 'Hello...!')
        self.assertEqual(truncate_with_ellipsis('Hello world!', 5, ellipsis=''), 'Hello')
        self.assertEqual(truncate_with_ellipsis('Hello world!', 5, ellipsis='...'), 'Hello...')

    def test_truncate_with_ellipsis_with_ellipsis_and_blank_disabled(self):
        self.assertEqual(truncate_with_ellipsis('Hello world! ', 6, ellipsis='...!', del_blank=False), 'Hello ...!')
        self.assertEqual(truncate_with_ellipsis('Hello world! ', 13, ellipsis='...!', del_blank=False), 'Hello world! ')


if __name__ == '__main__':
    unittest.main()



import re
from typing import List
from unidecode import unidecode


def highlight_text(
        text: str,
        words: List[str],
        start_tag: str = '<span>',
        end_tag: str = '</span>',
        case_insensitive: bool = False,
        accents_insensitive: bool = False,
        word_boundaries: bool = False
) -> str:
    """
    Highlights specified words in a given text by wrapping them with start_tag and end_tag.

    Args:
        text (str): The input text to be processed.
        words (list): A list of words to be highlighted.
        start_tag (str, optional): The starting HTML tag to wrap around the highlighted words. Defaults to '<span>'.
        end_tag (str, optional): The ending HTML tag to wrap around the highlighted words. Defaults to '</span>'.
        case_insensitive (bool, optional): If True, performs case-insensitive matching. Defaults to False.
        accents_insensitive (bool, optional): If True, performs accent-insensitive matching. Defaults to False.
        word_boundaries (bool, optional): If True, matches whole words only. Defaults to True.

    Returns:
        str: The text with the specified words highlighted using the provided tags.

    Example:
        >>> highlight_text('Hello world! This is a sample text.', ['hello', 'sample'])
        '<span>Hello</span> world! This is a <span>sample</span> text.'
    """
    positions = []

    replaced_text = text
    search_text = text
    if accents_insensitive:
        search_text = unidecode(text)
        words = [unidecode(word) for word in words]

    flags = re.IGNORECASE if case_insensitive else 0

    for word in words:
        pattern = re.compile(re.escape(word), flags)

        if word_boundaries:
            pattern = re.compile(r'\b' + pattern.pattern + r'\b', flags)

        matches = pattern.finditer(search_text)

        for match in matches:
            start, end = match.start(), match.end()

            # Check if the current match overlaps with any existing positions
            is_overlapping = False
            overlapping_positions = []

            for pos_start, pos_end in positions:
                if start <= pos_end and end >= pos_start:
                    is_overlapping = True
                    overlapping_positions.append((pos_start, pos_end))

            if is_overlapping:
                # Find the smallest start and largest end among the overlapping positions
                min_start = min(start, min([pos_start for pos_start, _ in overlapping_positions]))
                max_end = max(end, max([pos_end for _, pos_end in overlapping_positions]))

                # Remove the overlapping positions from the list
                for pos in overlapping_positions:
                    positions.remove(pos)

                # Add the aggregated position to the list
                positions.append((min_start, max_end))
            else:
                positions.append((start, end))

    # Replace words in reverse order to avoid modifying newly added <span> tags
    positions.sort(key=lambda pos: pos[0], reverse=True)
    for start, end in positions:
        replaced_text = replaced_text[:start] + start_tag + replaced_text[start:end] + end_tag + replaced_text[end:]

    return replaced_text



import unittest
from datetime import date

from vha_toolbox import (
    get_last_day_of_year,
    get_first_day_of_year,
    get_first_day_of_month,
    get_last_day_of_month,
    get_last_day_of_quarter,
    get_first_day_of_quarter,
    is_renewal_due,
    next_renewal_date
)


class DateUtilsTestCase(unittest.TestCase):
    def test_get_first_day_of_year(self):
        dt = date(2023, 6, 15)
        result = get_first_day_of_year(dt)
        self.assertEqual(result, date(2023, 1, 1))

    def test_get_last_day_of_year(self):
        dt = date(2023, 6, 15)
        result = get_last_day_of_year(dt)
        self.assertEqual(result, date(2023, 12, 31))

    def test_get_first_day_of_quarter(self):
        dt = date(2023, 6, 15)
        result = get_first_day_of_quarter(dt)
        self.assertEqual(result, date(2023, 4, 1))

    def test_get_last_day_of_quarter(self):
        dt = date(2023, 6, 15)
        result = get_last_day_of_quarter(dt)
        self.assertEqual(result, date(2023, 6, 30))

    def test_get_first_day_of_month(self):
        dt = date(2023, 6, 15)
        result = get_first_day_of_month(dt)
        self.assertEqual(result, date(2023, 6, 1))

    def test_get_first_day_of_month_2(self):
        dt = date(2024, 2, 10)
        result = get_first_day_of_month(dt)
        self.assertEqual(result, date(2024, 2, 1))

    def test_get_last_day_of_month(self):
        dt = date(2023, 6, 15)
        result = get_last_day_of_month(dt)
        self.assertEqual(result, date(2023, 6, 30))

    def test_get_last_day_of_month_2(self):
        dt = date(2024, 2, 10)
        result = get_last_day_of_month(dt)
        self.assertEqual(result, date(2024, 2, 29))

    def test_get_last_day_of_month_3(self):
        dt = date(2021, 2, 10)
        result = get_last_day_of_month(dt)
        self.assertEqual(result, date(2021, 2, 28))

    # Existing tests
    def test_is_renewal_due_same_day(self):
        self.assertFalse(is_renewal_due(date(2023, 6, 15), date(2023, 6, 15)))

    def test_is_renewal_due_next_day(self):
        self.assertFalse(is_renewal_due(date(2023, 6, 15), date(2023, 6, 16)))

    def test_is_renewal_due_next_month(self):
        self.assertTrue(is_renewal_due(date(2023, 6, 15), date(2023, 7, 15)))

    def test_is_renewal_due_next_year(self):
        self.assertTrue(is_renewal_due(date(2023, 6, 15), date(2024, 7, 15)))

    def test_leap_year_feb_29(self):
        self.assertTrue(is_renewal_due(date(2020, 2, 29), date(2021, 2, 28)))
        self.assertFalse(is_renewal_due(date(2020, 2, 29), date(2021, 3, 1)))

    def test_non_leap_year_feb_28(self):
        self.assertFalse(is_renewal_due(date(2023, 2, 28), date(2024, 2, 27)))
        self.assertTrue(is_renewal_due(date(2023, 2, 28), date(2024, 2, 28)))

    def test_30_day_month_to_31_day_month(self):
        self.assertTrue(is_renewal_due(date(2023, 4, 30), date(2024, 4, 30)))
        self.assertFalse(is_renewal_due(date(2023, 4, 30), date(2024, 5, 1)))

    def test_31_day_month_to_30_day_month(self):
        self.assertFalse(is_renewal_due(date(2023, 5, 31), date(2024, 5, 30)))
        self.assertTrue(is_renewal_due(date(2023, 5, 31), date(2024, 5, 31)))

    def test_past_date(self):
        self.assertTrue(is_renewal_due(date(2020, 1, 1), date(2024, 1, 1)))

    def test_future_date(self):
        self.assertFalse(is_renewal_due(date(2025, 1, 1), date(2024, 1, 1)))

    def test_standard_month_transition(self):
        self.assertEqual(next_renewal_date(date(2023, 5, 15)), date(2023, 6, 15))

    def test_end_of_year_transition(self):
        self.assertEqual(next_renewal_date(date(2023, 12, 15)), date(2024, 1, 15))

    def test_leap_year_feb(self):
        self.assertEqual(next_renewal_date(date(2024, 2, 29)), date(2024, 3, 29))

    def test_non_leap_year_feb(self):
        self.assertEqual(next_renewal_date(date(2023, 2, 28)), date(2023, 3, 28))

    def test_30_day_month_transition(self):
        self.assertEqual(next_renewal_date(date(2023, 4, 30)), date(2023, 5, 30))

    def test_31_day_month_transition_to_30_day_month(self):
        self.assertEqual(next_renewal_date(date(2023, 5, 31)), date(2023, 6, 30))

    def test_multiple_months_advance(self):
        self.assertEqual(next_renewal_date(date(2023, 1, 15), 3), date(2023, 4, 15))

    def test_end_of_month_feb_to_non_leap_march(self):
        self.assertEqual(next_renewal_date(date(2023, 1, 31), 1), date(2023, 2, 28))

    def test_end_of_month_feb_to_leap_march(self):
        self.assertEqual(next_renewal_date(date(2024, 1, 31), 1), date(2024, 2, 29))

    def test_transition_over_a_year(self):
        self.assertEqual(next_renewal_date(date(2023, 5, 15), 12), date(2024, 5, 15))

    def test_multiple_months_advance_standard(self):
        # Test advancing multiple months from a standard date
        self.assertEqual(next_renewal_date(date(2023, 1, 15), 6), date(2023, 7, 15))

    def test_end_of_year_multiple_months_advance(self):
        # Test advancing over the end of the year
        self.assertEqual(next_renewal_date(date(2023, 10, 31), 4), date(2024, 2, 29))

    def test_leap_year_multiple_months_advance(self):
        # Test advancing multiple months including a leap year February
        self.assertEqual(next_renewal_date(date(2023, 12, 31), 14), date(2025, 2, 28))

    def test_non_leap_year_february_advance(self):
        # Test advancing from a non-leap year February
        self.assertEqual(next_renewal_date(date(2023, 2, 28), 12), date(2024, 2, 28))

    def test_long_term_advance(self):
        # Test advancing a significant number of months
        self.assertEqual(next_renewal_date(date(2023, 1, 31), 36), date(2026, 1, 31))

    def test_end_of_month_transition_through_leap_year(self):
        # Test advancing from the end of a month through a leap year
        self.assertEqual(next_renewal_date(date(2023, 1, 31), 13), date(2024, 2, 29))

    def test_end_of_month_transition_non_leap_year(self):
        # Test advancing from the end of a month through a non-leap year
        self.assertEqual(next_renewal_date(date(2023, 1, 31), 1), date(2023, 2, 28))


if __name__ == '__main__':
    unittest.main()



from typing import List, Union


class SQL:

    def __init__(self, table_name: str, columns: List[str]):
        """
        A helper class for building SQL queries.

        Args:
            table_name (str): The name of the table to query.
            columns (list): A list of columns to select.

        Examples:
            >>> sql = SQL('my_table', ['column_1', 'column_2'])
            >>> print(sql)
            SELECT column_1, column_2 FROM my_table
        """
        self.table_name = table_name
        self.columns = columns
        self.with_statements = []
        self.join_statements = []
        self.where_statement = None
        self.order_by = None
        self.limit = None
        self._sql = None

    def add_join(self, join_statement: str):
        """
        Adds a JOIN statement to the query.

        Args:
            join_statement (str): The JOIN statement to add.

        Examples:
            >>> sql = SQL('my_table', ['column_1', 'column_2'])
            >>> sql.add_join('JOIN other_table ON my_table.id = other_table.id')
            >>> print(sql)
            SELECT column_1, column_2 FROM my_table JOIN other_table ON my_table.id = other_table.id
        """
        self.join_statements.append(join_statement)
        self._sql = None

    def add_with(self, name: str, statement: str):
        """
        Adds a WITH statement to the query.

        Args:
            name (str): The name of the WITH statement.
            statement (str): The statement to add.

        Examples:
            >>> sql = SQL('my_table', ['column_1', 'column_2'])
            >>> sql.add_with('with_1', 'SELECT * FROM other_table')
            >>> sql.add_with('with_2', 'SELECT * FROM other_table')
            >>> print(sql)
            WITH with_1 AS (SELECT * FROM other_table)
            WITH with_2 AS (SELECT * FROM other_table)
            SELECT column_1, column_2 FROM my_table
        """
        for x in self.with_statements:
            if x[0] == name:
                raise ValueError(f'Name {name} already exists.')
        self.with_statements.append((name, statement))
        self._sql = None

    def remove_with(self, name: str):
        """
        Removes a WITH statement from the query.

        Args:
            name (str): The name of the WITH statement to remove.
        """
        self.with_statements = [x for x in self.with_statements if x[0] != name]
        self._sql = None

    def set_where(self, where_statement: str):
        """
        Sets the WHERE statement for the query.

        Args:
            where_statement (str): The WHERE statement to set.

        Examples:
            >>> sql = SQL('my_table', ['column_1', 'column_2'])
            >>> sql.set_where('column_1 > 5')
            >>> print(sql)
            SELECT column_1, column_2 FROM my_table WHERE column_1 > 5
        """
        self.where_statement = where_statement
        self._sql = None

    def set_limit(self, limit: Union[int, None]):
        """
        Sets the LIMIT statement for the query.

        Args:
            limit (int | None): The LIMIT statement to set.

        Examples:
            >>> sql = SQL('my_table', ['column_1', 'column_2'])
            >>> sql.set_limit(10)
            >>> print(sql)
            SELECT column_1, column_2 FROM my_table LIMIT 10
        """
        self.limit = limit
        self._sql = None

    def set_order_by(self, order_by: str):
        """
        Sets the ORDER BY statement for the query.

        Args:
            order_by (str): The ORDER BY statement to set.

        Examples:
            >>> sql = SQL('my_table', ['column_1', 'column_2'])
            >>> sql.set_order_by('column_1 DESC')
            >>> print(sql)
            SELECT column_1, column_2 FROM my_table ORDER BY column_1 DESC
        """
        self.order_by = order_by
        self._sql = None

    def _generate_sql(self):
        if self._sql is None:
            self._sql = ''
            for name, statement in self.with_statements:
                self._sql += f'WITH {name} AS ({statement})\n'
            self._sql += f'SELECT {", ".join(self.columns)} FROM {self.table_name}'
            if self.join_statements:
                for join_statement in self.join_statements:
                    self._sql += f' {join_statement}'
            if self.where_statement is not None:
                self._sql += f' WHERE {self.where_statement}'
            if self.order_by is not None:
                self._sql += f' ORDER BY {self.order_by}'
            if self.limit is not None:
                self._sql += f' LIMIT {self.limit}'
        return self._sql

    def __str__(self):
        """
        Returns the SQL query.
        """
        return self._generate_sql()



from typing import List


def generate_table(
        rows: List[List[str]],
        header: List[str] = None,
        pretty: bool = False
) -> str:
    """
    Convert a list of rows to an HTML table with a header row.

    Args:
        rows (list of lists): The rows of data for the table.
        header (list): The header row for the table.
        pretty (bool): Whether to add newlines and indentation to the HTML table. Defaults to False.

    Returns:
        str: The HTML representation of the table.

    Example:
        >>> generate_table([['a', 'b'], ['c', 'd']])
        '<table><tr><th>a</th><th>b</th></tr><tr><td>c</td><td>d</td></tr></table>'
        >>> generate_table([['a', 'b'], ['c', 'd']], header=['x', 'y'])
        '<table><tr><th>x</th><th>y</th></tr><tr><td>a</td><td>b</td></tr><tr><td>c</td><td>d</td></tr></table>'
    """
    if header is None:
        header = []

    table = '<table>'

    if pretty:
        if header:
            table += '\n    <tr>'
            for column in header:
                table += f'\n        <th>{column}</th>'
            table += '\n    '
            table += '</tr>'

        for row in rows:
            table += '\n    <tr>'
            for column in row:
                table += f'\n        <td>{column}</td>'
            table += '\n    </tr>'
        table += '\n'
    else:
        if header:
            table += '<tr>'
            for column in header:
                table += f'<th>{column}</th>'
            table += '</tr>'

        for row in rows:
            table += '<tr>'
            for column in row:
                table += f'<td>{column}</td>'
            table += '</tr>'

    table += '</table>'
    return table



from typing import Union

COUNTRY_CODES = {
    "000-019": "United States and Canada",
    "020-029": "Restricted distribution (MO defined)",
    "030-039": "United States drugs (MO defined)",
    "040-049": "Restricted distribution (MO defined)",
    "050-059": "Coupons",
    "060-099": "United States and Canada",
    "100-139": "United States",
    "200-299": "Restricted distribution (MO defined)",
    "300-379": "France and Monaco",
    "380": "Bulgaria",
    "383": "Slovenia",
    "385": "Croatia",
    "387": "Bosnia and Herzegovina",
    "389": "Montenegro",
    "400-440": "Germany",
    "450-459": "Japan",
    "460-469": "Russia",
    "470": "Kyrgyzstan",
    "471": "Taiwan",
    "474": "Estonia",
    "475": "Latvia",
    "476": "Azerbaijan",
    "477": "Lithuania",
    "478": "Uzbekistan",
    "479": "Sri Lanka",
    "480": "Philippines",
    "481": "Belarus",
    "482": "Ukraine",
    "484": "Moldova",
    "485": "Armenia",
    "486": "Georgia",
    "487": "Kazakhstan",
    "489": "Hong Kong",
    "490-499": "Japan",
    "500-509": "United Kingdom",
    "520-521": "Greece",
    "528": "Lebanon",
    "529": "Cyprus",
    "530": "Albania",
    "531": "Macedonia",
    "535": "Malta",
    "539": "Ireland",
    "540-549": "Belgium and Luxembourg",
    "560": "Portugal",
    "569": "Iceland",
    "570-579": "Denmark, Faroe Islands and Greenland",
    "590": "Poland",
    "594": "Romania",
    "599": "Hungary",
    "600-601": "South Africa",
    "603": "Ghana",
    "604": "Senegal",
    "608": "Bahrain",
    "609": "Mauritius",
    "611": "Morocco",
    "613": "Algeria",
    "616": "Kenya",
    "618": "Ivory Coast",
    "619": "Tunisia",
    "621": "Syria",
    "622": "Egypt",
    "624": "Libya",
    "625": "Jordan",
    "626": "Iran",
    "627": "Kuwait",
    "628": "Saudi Arabia",
    "629": "United Arab Emirates",
    "640-649": "Finland",
    "690-695": "China",
    "700-709": "Norway",
    "729": "Israel",
    "730-739": "Sweden",
    "740": "Guatemala",
    "741": "El Salvador",
    "742": "Honduras",
    "743": "Nicaragua",
    "744": "Costa Rica",
    "745": "Panama",
    "746": "Dominican Republic",
    "750": "Mexico",
    "754-755": "Canada",
    "759": "Venezuela",
    "760-769": "Switzerland and Liechtenstein",
    "770": "Colombia",
    "773": "Uruguay",
    "775": "Peru",
    "777": "Bolivia",
    "779": "Argentina",
    "780": "Chile",
    "784": "Paraguay",
    "785": "Peru",
    "786": "Ecuador",
    "789-790": "Brazil",
    "800-839": "Italy, San Marino and Vatican City",
    "840-849": "Spain and Andorra",
    "850": "Cuba",
    "858": "Slovakia",
    "859": "Czech Republic",
    "860": "Serbia",
    "865": "Mongolia",
    "867": "North Korea",
    "868-869": "Turkey",
    "870-879": "Netherlands",
    "880": "South Korea",
    "884": "Cambodia",
    "885": "Thailand",
    "888": "Singapore",
    "890": "India",
    "893": "Vietnam",
    "896": "Pakistan",
    "899": "Indonesia",
    "900-919": "Austria",
    "930-939": "Australia",
    "940-949": "New Zealand",
    "950": "GS1 Global Office: Special applications",
    "951": "EPCglobal: Special applications",
    "955": "Malaysia",
    "958": "Macau",
    "960-969": "GS1 UK: GTIN-8 allocations",
    "977": "Serial publications (ISSN)",
    "978-979": "Bookland (ISBN)",
    "980": "Refund receipts",
    "981-982": "Common Currency Coupons",
    "990-999": "Coupons",
}


class EAN:
    """
    A class used to represent an EAN-13 (either EAN-13 or EAN-8).
    """
    def __init__(self, ean):
        """
        Args:
            ean (str): The EAN-13 to validate and format.

        Examples:
            >>> EAN('9783161484100')
            EAN(9783161484100)
            >>> EAN('978-3-16-148410-0')
            EAN(9783161484100)

        Raises:
            ValueError: If the EAN-13 is invalid (either format is incorrect or check digit is incorrect).
        """
        self.ean = self._sanitize_and_normalize_ean(ean)
        is_valid = self.is_valid()
        if not is_valid:
            raise ValueError("Invalid EAN, check digit is incorrect")

    def _sanitize_and_normalize_ean(self, ean):
        # Sanitize and normalize the provided EAN-13
        ean = ean.replace("-", "").replace(" ", "").lower()
        normalize = ''.join(char for char in ean if char.isdigit())
        # if len(normalize) == 13 or len(normalize) == 8:
        if len(normalize) == 13:
            return normalize
        else:
            raise ValueError("Invalid EAN, format is incorrect")

    def _break_down_ean_13(self) -> list[str]:
        """
        Breaks down an EAN-13 into its parts.

        Returns:
            list[str]: A list of strings containing the EAN-13 parts.
        """
        parts = [
            "Country: " + self.ean[:3],
            "Manufacturer/Product: " + self.ean[3:12],
            "Check digit: " + self.ean[12]
        ]

        return parts

    def _break_down_ean_8(self) -> list[str]:
        """
        Breaks down an EAN-8 into its parts.

        Returns:
            list[str]: A list of strings containing the EAN-8 parts.
        """
        parts = [
            "Country: " + self.ean[:3],
            "Manufacturer/Product: " + self.ean[3:7],
            "Check digit: " + self.ean[7]
        ]

        return parts

    def break_down_ean(self) -> list[str]:
        """
        Breaks down an EAN-13 or EAN-8 into its parts.

        Returns:
            list[str]: A list of strings containing the EAN-13 or EAN-8 parts.
        """
        if len(self.ean) == 13:
            return self._break_down_ean_13()
        elif len(self.ean) == 8:
            return self._break_down_ean_8()
        else:
            raise ValueError("Invalid EAN")

    def is_valid(self) -> bool:
        """
        Validates an EAN-13 by checking if the check digit is correct.

        Returns:
            bool: True if the EAN-13 is valid, False otherwise.

        Examples:
            >>> EAN('9783161484100').is_valid()
            True
            >>> EAN('9783161484105').is_valid()
            False
        """
        check_digit = sum(int(self.ean[i]) * (1 if i % 2 == 0 else 3) for i in range(12))
        check_digit = (10 - (check_digit % 10)) % 10
        return check_digit == int(self.ean[-1])

    def format(self) -> str:
        """
        Formats an EAN-13 by adding dashes.

        Returns:
            str: The formatted EAN-13.
        """
        return self.ean

    def get_country_name(self) -> str:
        """
        Gets the country name of the EAN-13.

        Returns:
            str: The country name of the EAN-13.
        """
        country_code = self.ean[:3]
        for key in COUNTRY_CODES:
            codes = key.split("-")
            if len(codes) == 1 and codes[0] == country_code:
                return COUNTRY_CODES[key]
            elif len(codes) != 1 and int(codes[0]) <= int(country_code) <= int(codes[1]):
                return COUNTRY_CODES[key]
        return "Unknown"

    def __str__(self):
        return self.format()

    def __repr__(self):
        return f"EAN({self.format()})"

    def __eq__(self, other: Union[object, str]) -> bool:
        """
        Compares two EAN-13s.

        Args:
            other (EAN or str): The EAN-13 to compare to.

        Returns:
            bool: True if the EAN-13s are equal, False otherwise.

        Examples:
            >>> EAN('9783161484100') == EAN('9783161484100')
            True
            >>> EAN('9783161484100') == '9783161484100'
            True
        """
        if isinstance(other, EAN):
            return self.ean == other.ean
        elif isinstance(other, str):
            return self.ean == other
        else:
            return False



import unittest

from vha_toolbox import EAN


class EanTestCase(unittest.TestCase):
    def test_ean_1(self):
        ean = EAN('5449000021199')
        result = ean.break_down_ean()
        self.assertEqual(result, [
            'Country: 544', 'Manufacturer/Product: 900002119', 'Check digit: 9'
        ])
        result = ean.format()
        self.assertEqual(result, '5449000021199')

    def test_ean_2(self):
        ean = EAN('9783161484100')
        result = ean.break_down_ean()
        self.assertEqual(result, [
            'Country: 978', 'Manufacturer/Product: 316148410', 'Check digit: 0'
        ])
        result = ean.format()
        self.assertEqual(result, '9783161484100')

    def test_invalid_ean(self):
        with self.assertRaises(ValueError):
            EAN('12345678901234')

    def test_equal(self):
        ean = EAN('5449000021199')
        self.assertFalse(ean == EAN('9783161484100'))
        self.assertTrue(ean == EAN('5449000021199'))

    def test_equal_str(self):
        ean = EAN('5449000021199')
        self.assertFalse(ean == '9783161484100')
        self.assertTrue(ean == '5449000021199')

    def test_country_name(self):
        isbn = EAN('9781861978769')
        self.assertEqual(isbn.get_country_name(), 'Bookland (ISBN)')
        isbn = EAN('5449000054227')
        self.assertEqual(isbn.get_country_name(), 'Belgium and Luxembourg')


if __name__ == '__main__':
    unittest.main()



from .data_manipulation import *
from .date_utils import *
from .ean import *
from .excel_column_conversion import *
from .file_hash_utils import *
from .format_size import *
from .frame_ratio_utils import *
from .highlight_text import *
from .isbn import *
from .magnet_link import *
from .sql_helper import *
from .string_manipulation import *
from .table_generator import *



import unittest

from vha_toolbox import anonymize_sentence


class AnonymizeSentenceTestCase(unittest.TestCase):

    def test_anonymize_sentence(self):
        sentence = 'Hello World'
        expected_anonymized_sentence = '***** *****'
        anonymized_sentence = anonymize_sentence(sentence)
        self.assertEqual(anonymized_sentence, expected_anonymized_sentence)

    def test_anonymize_sentence_with_special_character(self):
        sentence = 'Alice is 28 years old.'
        expected_anonymized_sentence = '***** ** ** ***** ***.'
        anonymized_sentence = anonymize_sentence(sentence)
        self.assertEqual(anonymized_sentence, expected_anonymized_sentence)

    def test_anonymize_sentence_with_special_character_2(self):
        sentence = 'This is a sample sentence with some 123 numbers and special characters!'
        expected_anonymized_sentence = '**** ** * ****** ******** **** **** *** ******* *** ******* **********!'
        anonymized_sentence = anonymize_sentence(sentence)
        self.assertEqual(anonymized_sentence, expected_anonymized_sentence)

    def test_anonymize_sentence_with_anonymize_character(self):
        sentence = 'Alice is 28 years old.'
        expected_anonymized_sentence = 'AAAAA AA AA AAAAA AAA.'
        anonymized_sentence = anonymize_sentence(sentence, anonymized_char='A')
        self.assertEqual(anonymized_sentence, expected_anonymized_sentence)

    # def test_anonymize_sentence_keep_partial_word(self):
    #     sentence = 'Alice is 28 years old.'
    #     expected_anonymized_sentence = 'A***e i* 2* y***s o**.'
    #     anonymized_sentence = anonymize_sentence(sentence, keep_partial_word=True)
    #     self.assertEqual(anonymized_sentence, expected_anonymized_sentence)
    #
    # def test_anonymize_sentence_keep_partial_word_2(self):
    #     sentence = 'Alice is 28 years old.'
    #     expected_anonymized_sentence = 'A**** i* 2* y**** o**.'
    #     anonymized_sentence = anonymize_sentence(sentence, keep_partial_word=True, erase_ratio=1)
    #     self.assertEqual(anonymized_sentence, expected_anonymized_sentence)


if __name__ == '__main__':
    unittest.main()



from typing import Union


class ISBN:
    """
    A class used to represent an ISBN (either ISBN-13 or ISBN-10).
    """
    def __init__(self, isbn):
        """
        Args:
            isbn (str): The ISBN to validate and format.

        Examples:
            >>> ISBN('978-1-86197-876-9')
            ISBN(978-1-86197-876-9)
            >>> ISBN('0-306-40615-2')
            ISBN(0-306-40615-2)
            >>> ISBN('9780306406157')
            ISBN(9-780-306-40615-7)

        Raises:
            ValueError: If the ISBN is invalid (either format is incorrect or check digit is incorrect).
        """
        self.isbn = self._sanitize_and_normalize_isbn(isbn)
        is_valid = self.is_valid()
        if not is_valid:
            raise ValueError("Invalid ISBN, check digit is incorrect")

    def _sanitize_and_normalize_isbn(self, isbn):
        # Sanitize and normalize the provided ISBN
        isbn = isbn.replace("-", "").replace(" ", "").upper()
        normalize = ''.join(char for char in isbn if char.isdigit() or char == 'X')
        if len(normalize) == 10 or len(normalize) == 13:
            return normalize
        else:
            raise ValueError("Invalid ISBN, format is incorrect")

    def _break_down_isbn_13(self) -> list[str]:
        """
        Breaks down an ISBN-13 into its parts.

        Returns:
            list[str]: A list of strings containing the ISBN-13 parts.
        """
        parts = [
            "Prefix: " + self.isbn[:3],
            "Registration group: " + self.isbn[3],
            "Registrant: " + self.isbn[4:9],
            "Publication: " + self.isbn[9:12],
            "Check digit: " + self.isbn[12]
        ]

        return parts

    def _break_down_isbn_10(self) -> list[str]:
        """
        Breaks down an ISBN-10 into its parts.

        Returns:
            list[str]: A list of strings containing the ISBN-10 parts.
        """
        parts = [
            "Group: " + self.isbn[:2],
            "Publisher: " + self.isbn[2:6],
            "Title: " + self.isbn[6:9],
            "Check digit: " + self.isbn[-1]
        ]

        return parts

    def break_down_isbn(self) -> list[str]:
        """
        Breaks down an ISBN into its parts.

        Returns:
            list[str]: A list of strings containing the ISBN parts.

        Example:
            >>> ISBN('9780306406157').break_down_isbn()
            ['Prefix: 978', 'Registration group: 0', 'Registrant: 30640', 'Publication: 615', 'Check digit: 7']
            >>> ISBN('0306406152').break_down_isbn()
            ['Group: 03', 'Publisher: 0640', 'Title: 615', 'Check digit: 2']
        """
        if len(self.isbn) == 13:
            return self._break_down_isbn_13()
        elif len(self.isbn) == 10:
            return self._break_down_isbn_10()
        else:
            raise ValueError("Invalid ISBN")

    def _is_isbn_13_valid(self) -> bool:
        """
        Checks the validity of an ISBN-13, allowing for dashes and lowercase characters.

        Returns:
            bool: True if the ISBN-13 is valid, False otherwise.
        """
        # (a * 1 + b * 3 + c * 1 + d * 3 + e * 1 + f * 3 + g * 1 + h * 3 + i * 1 + j * 3 + k * 1 + l * 3) % 10
        check_sum = sum(int(digit) * (1 if index % 2 == 0 else 3) for index, digit in enumerate(self.isbn[:-1]))
        calculated_check_digit = (10 - (check_sum % 10)) % 10
        return calculated_check_digit == int(self.isbn[-1])

    def _is_isbn_10_valid(self) -> bool:
        """
        Checks the validity of an ISBN-10, allowing for dashes and lowercase characters.

        Returns:
            bool: True if the ISBN-10 is valid, False otherwise.
        """
        # (a * 1 + b * 2 + c * 3 + d * 4 + e * 5 + f * 6 + g * 7 + h * 8 + i * 9 + j * 10 + k * 11) % 11
        check_sum = sum((i + 1) * int(digit) if digit != 'X' else 10 for i, digit in enumerate(self.isbn[:-1]))
        return check_sum % 11 == int(self.isbn[-1]) if self.isbn[-1] != 'X' else check_sum % 11 == 10

    def is_valid(self) -> bool:
        """
        Checks the validity of an ISBN (either ISBN-13 or ISBN-10), allowing for dashes and lowercase characters.

        Returns:
            bool: True if the ISBN is valid, False otherwise.
        """
        if len(self.isbn) == 13:
            return self._is_isbn_13_valid()
        elif len(self.isbn) == 10:
            return self._is_isbn_10_valid()
        else:
            return False

    def format(self) -> str:
        """
        Formats an ISBN by adding dashes for better readability after normalizing the input.
    
        Returns:
            str: The formatted ISBN.
    
        Examples:
            >>> ISBN('9783161484105').format()
            '978-3-16-148410-5'
            >>> ISBN('316148410X').format()
            '3-16-148410-X'
            >>> ISBN('9781861978769').format()
            '978-1-86-197876-9'
        """
        if len(self.isbn) == 10:
            return f"{self.isbn[:1]}-{self.isbn[1:4]}-{self.isbn[4:9]}-{self.isbn[9]}"
        elif len(self.isbn) == 13:
            return f"{self.isbn[:3]}-{self.isbn[3:4]}-{self.isbn[4:6]}-{self.isbn[6:12]}-{self.isbn[12]}"

    def _generate_isbn_13_check_digit(self, prefix: str = "978") -> int:
        """
        Generates the check digit for an ISBN-13.

        Args:
            prefix (str, optional): The EAN-13 prefix to use for ISBN-10. Defaults to "978".

        Returns:
            int: The check digit.

        Examples:
            >>> ISBN('0306406152')._generate_isbn_13_check_digit()
            7
            >>> ISBN('0306406152')._generate_isbn_13_check_digit("979")
            7
            >>> ISBN('9783161484105')._generate_isbn_13_check_digit()
            5

        Raises:
            ValueError: If the prefix is invalid.
        """
        if prefix not in ("978", "979"):
            raise ValueError("Invalid prefix")

        check_sum = sum(int(digit) * (1 if index % 2 == 0 else 3) for index, digit in enumerate(prefix + self.isbn[:-1]))
        return (10 - (check_sum % 10)) % 10

    def _isbn_10_to_isbn_13(self, prefix: str = "978") -> object:
        """
        Converts an ISBN-10 to ISBN-13.

        Args:
            prefix (str, optional): The EAN-13 prefix to use for ISBN-10. Defaults to "978".

        Returns:
            ISBN: The converted ISBN-13.

        Examples:
            >>> ISBN('0306406152')._isbn_10_to_isbn_13()
            ISBN(978-0-30-640615-7)
            >>> ISBN('0306406152')._isbn_10_to_isbn_13("979")
            ISBN(979-0-30-640615-7)
            >>> ISBN('9783161484105')._isbn_10_to_isbn_13()
            ISBN(978-3-16-148410-5)

        Raises:
            ValueError: If the prefix is invalid.
        """
        if prefix not in ("978", "979"):
            raise ValueError("Invalid prefix")

        return ISBN(prefix + self.isbn[:-1] + str(self._generate_isbn_13_check_digit(prefix)))

    def to_isbn_13(self, prefix: str = "978") -> str:
        """
        Converts an ISBN-10 to ISBN-13.

        Args:
            prefix (str, optional): The EAN-13 prefix to use for ISBN-10. Defaults to "978".

        Returns:
            str: The converted ISBN-13.

        Examples:
            >>> ISBN('0306406152').to_isbn_13()
            '9780306406157'
            >>> ISBN('0306406152').to_isbn_13("979")
            '9790306406157'
            >>> ISBN('9783161484105').to_isbn_13()
            '9783161484105'
        """
        if prefix not in ("978", "979"):
            raise ValueError("Invalid prefix")

        if len(self.isbn) == 10:
            return self._isbn_10_to_isbn_13(prefix).format()
        elif len(self.isbn) == 13:
            return self.format()

    def to_ean_13(self, prefix: str = "978") -> str:
        """
        Converts an ISBN to EAN-13.

        Args:
            prefix (str, optional): The EAN-13 prefix to use for ISBN-10. Defaults to "978".

        Returns:
            str: The converted EAN-13.

        Examples
            >>> ISBN('0306406152').to_ean_13()
            '9780306406157'
            >>> ISBN('0306406152').to_ean_13("979")
            '9790306406157'
            >>> ISBN('9783161484105').to_ean_13()
            '9783161484105'
        """
        if prefix not in ("978", "979"):
            raise ValueError("Invalid prefix")

        if len(self.isbn) == 10:
            return self._isbn_10_to_isbn_13(prefix).isbn
        elif len(self.isbn) == 13:
            return self.isbn

    def __str__(self):
        return self.format()

    def __repr__(self):
        return f"ISBN({self.format()})"

    def __eq__(self, other: Union[object, str]) -> bool:
        """
        Compares two ISBNs.

        Args:
            other (ISBN or str): The other ISBN to compare to.

        Returns:
            bool: True if the ISBNs are equal, False otherwise.

        Examples:
            >>> ISBN('978-1-86197-876-9') == ISBN('978-1-86197-876-9')
            True
            >>> ISBN('978-1-86197-876-9') == '978-1-86197-876-9'
            True
        """
        if isinstance(other, ISBN):
            return self.isbn == other.isbn
        elif isinstance(other, str):
            return self.isbn == ISBN(other).isbn
        else:
            return False



import math


def format_readable_size(size: int, decimal_places: int = 1) -> str:
    """
    Format the byte size in a human-readable format. The result is rounded to the nearest decimal place.

    Args:
        size (int): The size of the file in bytes.
        decimal_places (int, optional): The number of decimal places to round the result to. Defaults to 1.

    Returns:
        str: The formatted file size.

    Example:
        >>> format_readable_size(123456789)
        '117.7 MB'
        >>> format_readable_size(123456789, decimal_places=2)
        '117.74 MB'
        >>> format_readable_size(1000000000000)
        '931.3 GB'

    Raises:
        ValueError: If the size is negative.
    """
    if size < 0:
        raise ValueError("Size cannot be negative.")

    if size == 0:
        return f"0.0 {'B'}"

    suffixes = ['B', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB', 'BB']
    magnitude = int(math.floor(math.log(size, 1024)))

    # Handle extremely large sizes
    if magnitude >= len(suffixes):
        magnitude = len(suffixes) - 1

    size /= math.pow(1024, magnitude)
    formatted_size = f"{size:.{decimal_places}f}"

    return f"{formatted_size} {suffixes[magnitude]}"


def to_bytes(size_str: str) -> int:
    """
    Convert a human-readable size to bytes. The value is an approximation.

    Args:
        size_str (str): The human-readable file size.

    Returns:
        int: The size of the file in bytes.

    Example:
        >>> to_bytes('117.7 MB')
        123417395
        >>> to_bytes('1.1 KB')
        1126

    Raises:
        ValueError: If the size is negative.
        ValueError: If the size format is invalid.
    """
    size_str = size_str.strip().lower()
    size, unit = size_str.split()

    size = float(size)

    if size < 0:
        raise ValueError("Size cannot be negative.")

    unit_multipliers = {
        'b': 1,
        'kb': 1024,
        'mb': 1024 ** 2,
        'gb': 1024 ** 3,
        'tb': 1024 ** 4,
        'pb': 1024 ** 5,
        'eb': 1024 ** 6,
        'zb': 1024 ** 7,
        'yb': 1024 ** 8,
        'bb': 1024 ** 9,
    }

    if unit not in unit_multipliers:
        raise ValueError("Invalid size format")

    size *= unit_multipliers[unit]

    return int(size)


def sort_human_readable_sizes(sizes_list: list, reverse: bool = False) -> list:
    """
    Sort a list of human-readable file sizes in ascending order.

    Args:
        sizes_list (list): A list of human-readable file sizes.
        reverse (bool, optional): If True, sort in descending order. Defaults to False.

    Returns:
        list: The sorted list of human-readable file sizes.

    Example:
        >>> sort_human_readable_sizes(['1.5 MB', '512 KB', '2 GB', '128 KB'])
        ['128 KB', '512 KB', '1.5 MB', '2 GB']
    """
    size_byte_tuples = [(to_bytes(size), size) for size in sizes_list]
    sorted_size_byte_tuples = sorted(size_byte_tuples, key=lambda x: x[0], reverse=reverse)
    sorted_sizes = [size for (_, size) in sorted_size_byte_tuples]

    return sorted_sizes



import math
from typing import Dict
import re


def truncate_with_ellipsis(
        input_string: str,
        max_length: int,
        ellipsis: str = '...',
        del_blank: bool = True,
) -> str:
    """
    Truncates a string with an ellipsis if it exceeds the maximum length.

    Args:
        input_string (str): The string to truncate.
        max_length (int): The maximum length of the string.
        ellipsis (str, optional): The ellipsis to use. Defaults to '...'.
        del_blank (bool, optional): Whether to delete trailing whitespace. Defaults to True.

    Returns:
        str: The truncated string.

    Examples:
        >>> truncate_with_ellipsis('Hello world!', 5)
        'Hello...'
        >>> truncate_with_ellipsis('Hello world! ', 6)
        'Hello...' # trailing whitespace is deleted by default (instead of 'Hello ...')

    Warning:
        This function is deprecated and will be removed in future versions. Use textwrap.shorten() instead.
    """
    if len(input_string) <= max_length:
        return input_string
    else:
        input_string = input_string[:max_length]
        if del_blank:
            input_string = input_string.rstrip()
        return input_string + ellipsis


def replace_multiple_substrings(
        input_string: str,
        replacements: Dict[str, str]
) -> str:
    """
    Replaces multiple substrings in a string based on a dictionary of replacements.

    Args:
        input_string (str): The original string to perform replacements on.
        replacements (dict): A dictionary where keys are substrings to be replaced and values are their replacements.

    Returns:
        str: The modified string with all replacements applied.

    Example:
        >>> replacements = {
            'apple': 'orange',
            'banana': 'grape',
            'cherry': 'melon'
        }
        >>>  original_string = 'I have an apple, a banana, and a cherry.'
        >>> replace_multiple_substrings(original_string, replacements)
        'I have an orange, a grape, and a melon.'
    """
    if not replacements:
        return input_string
    pattern = re.compile("|".join([re.escape(k) for k in sorted(replacements, key=len, reverse=True)]), flags=re.DOTALL)
    return pattern.sub(lambda x: replacements[x.group(0)], input_string)


def anonymize_sentence(
        input_string: str,
        # keep_partial_word: bool = False,
        anonymized_char: str = '*',
        # erase_ratio: float = 0.5
) -> str:
    """
    Anonymizes a sentence by replacing all words with a specified character.

    Args:
        input_string (str): The input string to be anonymized.
        anonymized_char (str, optional): The character to use for anonymization. Defaults to '*'.

    Returns:
        str: The anonymized string.

    Examples:
        >>> anonymize_sentence('Hello World')
        '***** *****'
        >>> anonymize_sentence('Alice is 28 years old.')
        '***** ** ** ***** ***.'
        >>> anonymize_sentence('This is a sample sentence with some 123 numbers and special characters!')
        '**** ** * ****** ******** **** **** *** ******* *** ******* **********!'
    """
    words = re.findall(r'\b\w+\b|\W', input_string)
    result = []

    for word in words:
        if word.isalnum():
            replacement = anonymized_char * len(word)
            word = replacement
        result.append(word)

    return ''.join(result)


def text_to_html(
        text: str,
        replacements: [str] = None
) -> str:
    """
    Converts a text string to HTML.

    Args:
        text:
        replacements:

    Returns:
        str: The converted HTML string.

    Examples:
        >>> text_to_html('Hello world!')
        '<p>Hello world!</p>'
        >>> text_to_html('Hello\nworld!')
        '<p>Hello</p><p>world!</p>'
    """
    if not text:
        return ''
    if replacements is None or not replacements:
        replacements = ['\n']

    for replacement in replacements:
        text = text.replace(replacement, '</p><p>')
    text = f'<p>{text}</p>'

    return text


def seconds_to_humantime(
        seconds: int,
        include_seconds: bool = True
) -> str:
    """
    Converts a number of seconds to a human-readable time format with years, months, days, hours, minutes, and seconds.

    Args:
        seconds (int): The number of seconds to convert.
        include_seconds (bool, optional): Whether to include seconds in the output. If the number is under 60 seconds,
        this parameter is ignored. Defaults to True.

    Returns:
        str: The human-readable time format.

    Examples:
        >>> seconds_to_humantime(3660)
        '1 hour 1 minute'
        >>> seconds_to_humantime(3660, include_seconds=False)
        '1 hour 1 minute'
        >>> seconds_to_humantime(60)
        '1 minute'
        >>> seconds_to_humantime(120)
        '2 minutes'
        >>> seconds_to_humantime(0)
        '0 seconds'
        >>> seconds_to_humantime(31536000 + 2592000 + 86400 + 3600 + 60 + 1)
        '1 year, 1 month, 1 day, 1 hour, 1 minute and 1 second'
    """
    if seconds < 0:
        raise ValueError("Input should be a non-negative integer.")

    seconds = int(seconds)

    intervals = (
        (31536000, "year"),  # 365 days
        (2592000, "month"),  # 30 days
        (86400, "day"),
        (3600, "hour"),
        (60, "minute"),
        (1, "second"),
    )

    parts = []
    for interval_seconds, unit in intervals:
        value, seconds = divmod(seconds, interval_seconds)
        if value:
            parts.append(f"{value} {unit}{'s' if value > 1 else ''}")

    if not include_seconds and parts and (parts[-1].endswith("seconds") or parts[-1].endswith("second")):
        parts.pop()

    if not parts:
        return "0 seconds"

    if len(parts) > 1:
        return ", ".join(parts[:-1]) + " and " + parts[-1]
    else:
        return parts[0]



import unittest

from vha_toolbox import get_position, get_letter


class ExcelColumnConversionTestCase(unittest.TestCase):
    def test_get_position(self):
        self.assertEqual(get_position('A'), 1)
        self.assertEqual(get_position('B'), 2)
        self.assertEqual(get_position('Z'), 26)
        self.assertEqual(get_position('AA'), 27)
        self.assertEqual(get_position('AB'), 28)
        self.assertEqual(get_position('BA'), 53)
        self.assertEqual(get_position('ZZ'), 702)
        self.assertEqual(get_position('AAA'), 703)
        self.assertEqual(get_position('ABC'), 731)

    def test_get_position_error(self):
        self.assertRaises(ValueError, get_position, '')
        self.assertRaises(ValueError, get_position, '1')
        self.assertRaises(ValueError, get_position, 'AA0')
        self.assertRaises(ValueError, get_position, None)

    def test_get_letter(self):
        self.assertEqual(get_letter(1), 'A')
        self.assertEqual(get_letter(2), 'B')
        self.assertEqual(get_letter(26), 'Z')
        self.assertEqual(get_letter(27), 'AA')
        self.assertEqual(get_letter(28), 'AB')
        self.assertEqual(get_letter(53), 'BA')
        self.assertEqual(get_letter(702), 'ZZ')
        self.assertEqual(get_letter(703), 'AAA')
        self.assertEqual(get_letter(731), 'ABC')

    def test_get_letter_error(self):
        self.assertRaises(ValueError, get_letter, 0)
        self.assertRaises(ValueError, get_letter, -1)
        self.assertRaises(ValueError, get_letter, 1.5)
        self.assertRaises(ValueError, get_letter, 'A')
        self.assertRaises(ValueError, get_letter, '')
        self.assertRaises(ValueError, get_letter, None)


if __name__ == '__main__':
    unittest.main()




def get_position(s: str) -> int:
    """
    Converts an Excel-style column label to a column number.

    Args:
        s (str): The Excel-style column label.

    Returns:
        int: The corresponding column number.

    Examples:
        >>> get_position('A')
        1
        >>> get_position('B')
        2
        >>> get_position('Z')
        26
        >>> get_position('AA')
        27
        >>> get_position('AB')
        28

    Raises:
        ValueError: If the input should be a non-empty string containing only letters.
    """
    if not s or not s.isalpha():
        raise ValueError("Input should be a non-empty string containing only letters.")

    pos = 0
    for letter in s:
        pos = pos * 26 + (ord(letter.lower()) - 96)
    return pos


def get_letter(i: int) -> str:
    """
    Converts a 1-relative column number to an Excel-style column label.

    Args:
        i (int): The 1-relative column number.

    Returns:
        str: The corresponding Excel-style column label.

    Examples:
        >>> get_letter(1)
        'A'
        >>> get_letter(2)
        'B'
        >>> get_letter(27)
        'AA'
        >>> get_letter(28)
        'AB'

    Raises:
        ValueError: If the input should be a positive integer.
    """
    if not isinstance(i, int) or i <= 0:
        raise ValueError("Input should be a positive integer.")

    letters = []
    while i > 0:
        i -= 1
        quot, rem = divmod(i, 26)
        letters.append(chr(rem + ord('A')))
        i = quot

    return ''.join(reversed(letters))



import unittest

from vha_toolbox import ISBN


class IsbnTestCase(unittest.TestCase):
    def test_valid_isbn_13(self):
        valid_isbn_13 = ISBN('978-1-86197-876-9')
        self.assertTrue(valid_isbn_13.is_valid())
        self.assertEqual(valid_isbn_13.break_down_isbn(), [
            'Prefix: 978', 'Registration group: 1', 'Registrant: 86197', 'Publication: 876', 'Check digit: 9'
        ])
        self.assertEqual(valid_isbn_13.format(), '978-1-86-197876-9')
        self.assertEqual(valid_isbn_13.to_isbn_13(), '978-1-86-197876-9')
        self.assertEqual(valid_isbn_13.to_ean_13(), '9781861978769')

    def test_valid_isbn_13_2(self):
        valid_isbn_13 = ISBN('9780306406157')
        self.assertTrue(valid_isbn_13.is_valid())
        self.assertEqual(valid_isbn_13.break_down_isbn(), [
            'Prefix: 978', 'Registration group: 0', 'Registrant: 30640', 'Publication: 615', 'Check digit: 7'
        ])
        self.assertEqual(valid_isbn_13.format(), '978-0-30-640615-7')
        self.assertEqual(valid_isbn_13.to_ean_13(), '9780306406157')

    def test_valid_isbn_10(self):
        valid_isbn_10 = ISBN('0-306-40615-2')
        self.assertTrue(valid_isbn_10.is_valid())
        self.assertEqual(valid_isbn_10.break_down_isbn(), [
            'Group: 03', 'Publisher: 0640', 'Title: 615', 'Check digit: 2'
        ])
        self.assertEqual(valid_isbn_10.format(), '0-306-40615-2')
        self.assertEqual(valid_isbn_10.to_isbn_13(), '978-0-30-640615-7')
        self.assertEqual(valid_isbn_10.to_ean_13(), '9780306406157')

    def test_valid_isbn_10_with_x(self):
        valid_isbn_10 = ISBN('0-9752298-0-X')
        self.assertTrue(valid_isbn_10.is_valid())
        self.assertEqual(valid_isbn_10.break_down_isbn(), [
            'Group: 09', 'Publisher: 7522', 'Title: 980', 'Check digit: X'
        ])
        self.assertEqual(valid_isbn_10.format(), '0-975-22980-X')
        self.assertEqual(valid_isbn_10.to_isbn_13(), '978-0-97-522980-4')
        self.assertEqual(valid_isbn_10.to_ean_13(), '9780975229804')

    def test_invalid_isbn(self):
        with self.assertRaises(ValueError):
            ISBN('3-16-148410-X1')  # Incorrect check digit
        with self.assertRaises(ValueError):
            ISBN('12345678901234')  # Invalid format

    def test_invalid_prefix(self):
        with self.assertRaises(ValueError):
            ISBN('0306406152').to_isbn_13("123")  # Invalid prefix
        with self.assertRaises(ValueError):
            ISBN('0306406152').to_ean_13("123")

    def test_string_representation(self):
        isbn_13 = ISBN('978-1-86197-876-9')
        self.assertEqual(str(isbn_13), '978-1-86-197876-9')
        self.assertEqual(repr(isbn_13), "ISBN(978-1-86-197876-9)")

    def test_valid_isbn_10_2(self):
        isbn = '2-9512774-2-3'
        isbn = ISBN(isbn)
        self.assertTrue(isbn.is_valid())
        self.assertEqual(isbn.break_down_isbn(), [
            'Group: 29', 'Publisher: 5127', 'Title: 742', 'Check digit: 3'
        ])
        self.assertEqual(isbn.format(), '2-951-27742-3')
        self.assertEqual(isbn.to_isbn_13(), '978-2-95-127742-7')
        self.assertEqual(isbn.to_ean_13(), '9782951277427')

    def test_equal(self):
        isbn_1 = ISBN('978-1-86197-876-9')
        self.assertTrue(isbn_1 == ISBN('978-1-86197-876-9'))
        self.assertFalse(isbn_1 == ISBN('2-9512774-2-3'))

    def test_equal_str(self):
        isbn_1 = ISBN('978-1-86197-876-9')
        self.assertTrue(isbn_1 == '978-1-86197-876-9')
        self.assertFalse(isbn_1 == '2-9512774-2-3')


if __name__ == '__main__':
    unittest.main()



from typing import Dict


def sort_dict_by_list(
        input_dict: Dict[str, any],
        order_list: list[str]
) -> Dict[str, any]:
    """
    Sorts a dictionary based on a custom order specified in a list.

    This function takes an input dictionary and a list of keys that represent the desired order
    of keys in the resulting dictionary. It returns a new dictionary with the keys from the input
    dictionary sorted according to the order specified in the 'order_list' list.

    Args:
        input_dict (dict): The input dictionary to be sorted.
        order_list (list): A list of keys specifying the desired order of keys.

    Returns:
        dict: A new dictionary with keys sorted based on the 'order_list' list.

    Examples:
        >>> unsorted_dict = {'b': 2, 'a': 1, 'c': 3}
        >>> order_list = ['a', 'b', 'c']
        >>> sort_dict_by_list(unsorted_dict, order_list)
        {'a': 1, 'b': 2, 'c': 3}

        >>> unsorted_dict = {'apple': 3, 'banana': 1, 'cherry': 2}
        >>> order_list = ['banana', 'cherry', 'apple']
        >>> sort_dict_by_list(unsorted_dict, order_list)
        {'banana': 1, 'cherry': 2, 'apple': 3}

    Raises:
        ValueError: If 'order_list' contains duplicate keys or keys not present in 'input_dict'.
    """
    # Check for duplicate keys or keys not present in the input dictionary
    if len(set(order_list)) != len(order_list) or not all(key in input_dict for key in order_list):
        raise ValueError("The 'order_list' list should contain unique keys present in the input dictionary.")

    # Create a sorted dictionary based on the 'order_list' list
    sorted_dict = {key: input_dict[key] for key in order_list}

    return sorted_dict



import re
from typing import List
from urllib.parse import quote


def create_magnet_link(
        info_hash: str,
        display_name: str,
        trackers: List[str],
) -> str:
    """
    Creates a magnet link from the provided info hash, display name, and trackers.

    Args:
        info_hash (str): The info hash of the torrent.
        display_name (str): The display name of the torrent.
        trackers (list): A list of trackers for the torrent.

    Returns:
        str: The magnet link.

    Example:
        >>> create_magnet_link('1234567890abcdef1234567890abcdef12345678', 'My Torrent', ['udp://tracker.example.com'])
        'magnet:?xt=urn:btih:1234567890abcdef1234567890abcdef12345678&dn=My+Torrent&tr=udp%3A%2F%2Ftracker.example.com'

    Raises:
        ValueError: If the info hash, display name, or list of trackers is empty.
    """
    if not info_hash:
        raise ValueError('The info hash cannot be empty.')
    if not display_name:
        raise ValueError('The display name cannot be empty.')
    if not trackers:
        raise ValueError('The list of trackers cannot be empty.')

    encoded_display_name = quote(display_name, safe='')
    magnet_link = f'magnet:?xt=urn:btih:{info_hash}&dn={encoded_display_name}'

    for tracker in trackers:
        encoded_tracker = quote(tracker, safe='')
        magnet_link += f'&tr={encoded_tracker}'

    return magnet_link



import unittest

from vha_toolbox import sort_dict_by_list


class SortDictTestCase(unittest.TestCase):
    def test_sort_dict_empty(self):
        input_dict = {}
        order_list = []
        result = sort_dict_by_list(input_dict, order_list)
        self.assertEqual(result, {})

    def test_sort_dict_single_key(self):
        input_dict = {'a': 1}
        order_list = ['a']
        result = sort_dict_by_list(input_dict, order_list)
        self.assertEqual(result, {'a': 1})

    def test_sort_dict_multiple_keys(self):
        input_dict = {'b': 2, 'a': 1, 'c': 3}
        order_list = ['a', 'b', 'c']
        result = sort_dict_by_list(input_dict, order_list)
        self.assertEqual(result, {'a': 1, 'b': 2, 'c': 3})

    def test_sort_dict_duplicate_keys(self):
        input_dict = {'b': 2, 'a': 1, 'c': 3}
        order_list = ['a', 'b', 'b', 'c']
        with self.assertRaises(ValueError):
            sort_dict_by_list(input_dict, order_list)

    def test_sort_dict_missing_keys(self):
        input_dict = {'b': 2, 'a': 1, 'c': 3}
        order_list = ['a', 'b', 'd', 'c']
        with self.assertRaises(ValueError):
            sort_dict_by_list(input_dict, order_list)


if __name__ == '__main__':
    unittest.main()



import math

RATIO_TOL = 0.05  # tolérance par défaut


def _ratio(w, h):
    return w / h if h else 0.0


def should_rotate_by_ratio(
    img,
    frame_img,
    frame_target_ratio: float,
    img_target_ratio: float,
    tol: float = RATIO_TOL,
    require_orientation_flip: bool = True,
):
    """
    Détermine si l'on doit tourner le cadre en fonction :

    - du ratio réel du cadre
    - du ratio réel de l'image
    - des ratios cibles souhaités (ex.: 2/3, 3/2, carré, etc.)
    - d'une tolérance
    - et éventuellement d'un changement de sens (portrait -> paysage)

    Args:
        img (PIL.Image.Image): L'image originale.
        frame_img (PIL.Image.Image): L'image du cadre.
        frame_target_ratio (float): Le ratio attendu pour le cadre (ex.: 2/3).
        img_target_ratio (float): Le ratio attendu pour l'image (ex.: 3/2).
        tol (float): Tolérance relative pour comparer les ratios.
        require_orientation_flip (bool): Si True, impose que l'un soit portrait et l'autre paysage.

    Returns:
        bool: True si le cadre doit être tourné, False sinon.
    """
    pw, ph = img.size
    fw, fh = frame_img.size

    img_r = _ratio(pw, ph)
    frame_r = _ratio(fw, fh)

    # cadre carré → jamais de rotation
    if math.isclose(frame_r, 1.0, rel_tol=tol):
        return False

    # vérifier le ratio du cadre
    frame_match = math.isclose(frame_r, frame_target_ratio, rel_tol=tol)

    # vérifier le ratio de l’image
    img_match = math.isclose(img_r, img_target_ratio, rel_tol=tol)

    if not (frame_match and img_match):
        return False

    # si orientation doit changer (portrait → paysage ou inverse)
    if require_orientation_flip:
        frame_is_portrait = fh > fw
        img_is_landscape = pw > ph
        orientation_ok = frame_is_portrait and img_is_landscape
        return orientation_ok

    return True



from calendar import monthrange
from datetime import date, timedelta


def get_first_day_of_year(dt: date = date.today()) -> date:
    """
    Get the first day of the year for the given date or the current date if not provided.

    Args:
        dt (date): The date for which to retrieve the first day of the year (default: current date).

    Returns:
        date: The first day of the year as a date object.

    Example:
        >>> get_first_day_of_year(date(2023, 6, 15))
        datetime.date(2023, 1, 1)
    """
    return date(dt.year, 1, 1)


def get_last_day_of_year(dt: date = date.today()) -> date:
    """
    Get the last day of the year for the given date or the current date if not provided.

    Args:
        dt (date): The date for which to retrieve the last day of the year (default: current date).

    Returns:
        date: The last day of the year as a date object.

    Example:
        >>> get_last_day_of_year(date(2023, 6, 15))
        datetime.date(2023, 12, 31)
    """
    return date(dt.year, 12, monthrange(dt.year, 12)[1])


def get_first_day_of_quarter(dt: date = date.today()) -> date:
    """
    Get the first day of the quarter for the given date or the current date if not provided.

    Args:
        dt (date): The date for which to retrieve the first day of the quarter (default: current date).

    Returns:
        date: The first day of the quarter as a date object.

    Example:
        >>> get_first_day_of_quarter(date(2023, 6, 15))
        datetime.date(2023, 4, 1)
    """
    return date(dt.year, (dt.month - 1) // 3 * 3 + 1, 1)


def get_last_day_of_quarter(dt: date = date.today()) -> date:
    """
    Get the last day of the quarter for the given date or the current date if not provided.

    Args:
        dt (date): The date for which to retrieve the last day of the quarter (default: current date).

    Returns:
        date: The last day of the quarter as a date object.

    Example:
        >>> get_last_day_of_quarter(date(2023, 6, 15))
        datetime.date(2023, 6, 30)
    """
    next_qt_yr = dt.year + (1 if dt.month > 9 else 0)
    next_qt_first_mo = (dt.month - 1) // 3 * 3 + 4
    next_qt_first_mo = 1 if next_qt_first_mo == 13 else next_qt_first_mo
    next_qt_first_dy = date(next_qt_yr, next_qt_first_mo, 1)
    return next_qt_first_dy - timedelta(days=1)


def get_first_day_of_month(dt: date = date.today()) -> date:
    """
    Get the first day of the month for the given date or the current date if not provided.

    Args:
        dt (date): The date for which to retrieve the first day of the month (default: current date).

    Returns:
        date: The first day of the month as a date object.

    Example:
        >>> get_first_day_of_month(date(2023, 6, 15))
        datetime.date(2023, 6, 1)
    """
    return date(dt.year, dt.month, 1)


def get_last_day_of_month(dt: date = date.today()) -> date:
    """
    Get the last day of the month for the given date or the current date if not provided.

    Args:
        dt (date): The date for which to retrieve the last day of the month (default: current date).

    Returns:
        date: The last day of the month as a date object.

    Example:
        >>> get_last_day_of_month(date(2023, 6, 15))
        datetime.date(2023, 6, 30)
    """
    return date(dt.year, dt.month, monthrange(dt.year, dt.month)[1])


def next_renewal_date(dt: date = date.today(), index: int = 1) -> date:
    """
    Get the next renewal date for the given date or the current date if not provided.

    Args:
        index (int): The number of months to add to the date (default: 1).
        dt (date): The date for which to retrieve the next renewal date (default: current date).

    Returns:
        date: The next renewal date as a date object.

    Example:
        >>> next_renewal_date(date(2023, 6, 15))
        datetime.date(2023, 7, 15)
        >>> next_renewal_date(date(2023, 6, 15), 2)
        datetime.date(2023, 8, 15)
        >>> next_renewal_date(date(2023, 10, 31), 4)
        datetime.date(2024, 2, 29)
    """
    year, month = dt.year, dt.month

    # Increment month and adjust year if necessary
    month += index
    while month > 12:
        month -= 12
        year += 1

    # Check if it's a leap year
    is_leap_year = year % 4 == 0 and (year % 100 != 0 or year % 400 == 0)

    # Adjust day for February in leap and non-leap years
    if month == 2:
        if is_leap_year and dt.day > 29:
            return date(year, 2, 29)
        elif not is_leap_year and dt.day > 28:
            return date(year, 2, 28)

    # Handle last day of month for 30 day months
    if month in [4, 6, 9, 11] and dt.day > 30:
        return date(year, month, 30)

    # Standard case
    return date(year, month, dt.day)


def is_renewal_due(dt_to_check: date, dt: date = date.today()) -> bool:
    """
    Check if the renewal date is due for the given date or the current date if not provided.
    This function accounts for different month lengths and leap years.

    Args:
        dt_to_check (date): The date to check.
        dt (date): The date to check against (default: current date).

    Returns:
        bool: True if the renewal date is due, False otherwise.

    Example:
        >>> is_renewal_due(date(2023, 6, 15), date(2023, 6, 15))
        False
        >>> is_renewal_due(date(2023, 6, 15), date(2023, 7, 15))
        True
        >>> is_renewal_due(date(2023, 6, 15), date(2023, 7, 16))
        False
        >>> is_renewal_due(date(2023, 6, 15), date(2024, 7, 15))
        True
        >>> is_renewal_due(date(2020, 2, 29), date(2021, 2, 28))
        True
        >>> is_renewal_due(date(2020, 5, 31), date(2021, 5, 30))
        True
    """
    # Check if today is passed or the same as the renewal date
    if dt_to_check >= dt:
        return False

    is_leap_year = dt.year % 4 == 0 and (dt.year % 100 != 0 or dt.year % 400 == 0)

    # Handle February for leap and non-leap years
    if dt_to_check.month == 2:
        if is_leap_year and dt.day == 29:
            return dt_to_check.day == 29
        elif not is_leap_year and dt.day == 28:
            return dt_to_check.day in [28, 29]

    # Handle renewal due for months with 30 days
    if dt_to_check.month in [4, 6, 9, 11] and dt_to_check.day == 31:
        return dt.day == 30

    # Default check for other cases
    return dt_to_check.day == dt.day



import pathlib

import setuptools

long_description = (pathlib.Path(__file__).parent.resolve() / 'README.md').read_text(encoding='utf-8')

setuptools.setup(
    name='vha-toolbox',
    version='0.0.13',
    author='Victor Hachard',
    author_email='31635811+VictorHachard@users.noreply.github.com',
    url='https://github.com/VictorHachard/vha-toolbox',
    description='My personal Python toolbox',
    long_description=long_description,
    long_description_content_type="text/markdown",
    project_urls={
        "Source": "https://github.com/VictorHachard/vha-toolbox",
        "Bug Tracker": "https://github.com/VictorHachard/vha-toolbox/issues"
    },
    license='MIT',
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.13",
        "Programming Language :: Python :: 3.14"
    ],
    packages=['vha_toolbox'],
    install_requires=[
        'Unidecode==1.4.0',
        'Pillow>=10.0.0',
    ],
)



import unittest
import tempfile
from pathlib import Path

from vha_toolbox import compute_file_hash, compute_stable_hash


class ComputeFileHashTestCase(unittest.TestCase):

    def test_hash_identical_files(self):
        """Two files with identical content must produce the same hash."""
        with tempfile.TemporaryDirectory() as tmp:
            p = Path(tmp)
            f1 = p / "file1.txt"
            f2 = p / "file2.txt"

            f1.write_text("Hello World")
            f2.write_text("Hello World")

            h1 = compute_file_hash(f1)
            h2 = compute_file_hash(f2)

            self.assertEqual(h1, h2)

    def test_hash_different_files(self):
        """Different content must produce a different hash."""
        with tempfile.TemporaryDirectory() as tmp:
            p = Path(tmp)
            f1 = p / "file1.txt"
            f2 = p / "file2.txt"

            f1.write_text("Hello World")
            f2.write_text("Other content")

            h1 = compute_file_hash(f1)
            h2 = compute_file_hash(f2)

            self.assertNotEqual(h1, h2)

    def test_hash_changes_with_extra_file(self):
        """Adding an extra file must change the resulting hash."""
        with tempfile.TemporaryDirectory() as tmp:
            p = Path(tmp)
            main = p / "main.txt"
            extra = p / "extra.txt"

            main.write_text("AAA")
            extra.write_text("BBB")

            h_main_only = compute_file_hash(main)
            h_with_extra = compute_file_hash(main, [extra])

            self.assertNotEqual(h_main_only, h_with_extra)

    def test_hash_changes_when_extra_file_changes(self):
        """Modifying an extra file must change the hash."""
        with tempfile.TemporaryDirectory() as tmp:
            p = Path(tmp)
            main = p / "main.txt"
            extra = p / "extra.txt"

            main.write_text("AAA")
            extra.write_text("BBB")

            h1 = compute_file_hash(main, [extra])

            extra.write_text("BBB modified")
            h2 = compute_file_hash(main, [extra])

            self.assertNotEqual(h1, h2)

    def test_stable_hash_with_multiple_extras(self):
        """Same files in the same order must always produce the same hash."""
        with tempfile.TemporaryDirectory() as tmp:
            p = Path(tmp)
            main = p / "main.txt"
            e1 = p / "a.txt"
            e2 = p / "b.txt"

            main.write_text("XXX")
            e1.write_text("111")
            e2.write_text("222")

            h1 = compute_file_hash(main, [e1, e2])
            h2 = compute_file_hash(main, [e1, e2])

            self.assertEqual(h1, h2)

    def test_short_hash_length(self):
        """The returned hash must be 8 characters long."""
        with tempfile.TemporaryDirectory() as tmp:
            p = Path(tmp)
            f = p / "x.txt"
            f.write_text("content")

            h = compute_file_hash(f)
            self.assertEqual(len(h), 8)

    def test_missing_file_raises(self):
        """A missing file must raise FileNotFoundError."""
        with self.assertRaises(FileNotFoundError):
            compute_file_hash("does_not_exist.txt")

    def test_same_values_same_hash(self):
        values1 = ["image1.png", 3000, 2]
        values2 = ["image1.png", 3000, 2]

        h1 = compute_stable_hash(values1)
        h2 = compute_stable_hash(values2)

        self.assertEqual(h1, h2)

    def test_different_values_different_hash(self):
        values1 = ["image1.png", 3000, 2]
        values2 = ["image1.png", 3000, 3]

        h1 = compute_stable_hash(values1)
        h2 = compute_stable_hash(values2)

        self.assertNotEqual(h1, h2)

    def test_dict_order_does_not_matter(self):
        v1 = {"x": 1, "y": 2}
        v2 = {"y": 2, "x": 1}

        h1 = compute_stable_hash(v1)
        h2 = compute_stable_hash(v2)

        self.assertEqual(h1, h2)

    def test_set_order_does_not_matter(self):
        v1 = {1, 2, 3}
        v2 = {3, 2, 1}

        h1 = compute_stable_hash(v1)
        h2 = compute_stable_hash(v2)

        self.assertEqual(h1, h2)

    def test_prefix_is_applied(self):
        values = ["a", "b", "c"]
        h = compute_stable_hash(values, prefix="row_")

        self.assertTrue(h.startswith("row_"))
        self.assertEqual(len(h) - len("row_"), 8)

    def test_custom_length(self):
        values = ["a", "b", "c"]
        h = compute_stable_hash(values, length=12)

        self.assertEqual(len(h), 12)

    def test_non_iterable_wrapped(self):
        h1 = compute_stable_hash("hello")
        h2 = compute_stable_hash(["hello"])

        self.assertEqual(h1, h2)

    def test_bytes_and_path_supported(self):
        data = b"\x01\x02\x03"
        path = Path("/some/path/image.avif")

        # Just ensure it does not raise and returns a string of correct length
        h = compute_stable_hash([data, path])
        self.assertIsInstance(h, str)
        self.assertEqual(len(h), 8)

    def test_nested_structures_stable(self):
        v1 = {
            "a": [1, 2, {"x": 10}],
            "b": {"k": {3, 4}},
        }
        v2 = {
            "b": {"k": {4, 3}},
            "a": [1, 2, {"x": 10}],
        }

        h1 = compute_stable_hash(v1)
        h2 = compute_stable_hash(v2)

        self.assertEqual(h1, h2)


if __name__ == '__main__':
    unittest.main()



import unittest

from vha_toolbox import create_magnet_link


class MagnetLinkTestCase(unittest.TestCase):
    def test_magnet_link(self):
        hash = '1234567890abcdef1234567890abcdef12345678'
        trackers = ['http://tracker1.com', 'http://tracker2.com']
        displayed_name = 'My Torrent'
        expected_output = 'magnet:?xt=urn:btih:1234567890abcdef1234567890abcdef12345678&dn=My%20Torrent&tr=http%3A%2F%2Ftracker1.com&tr=http%3A%2F%2Ftracker2.com'
        self.assertEqual(create_magnet_link(hash, displayed_name, trackers), expected_output)

    def test_magnet_link_with_empty_hash(self):
        hash = ''
        trackers = ['http://tracker1.com', 'http://tracker2.com']
        displayed_name = 'My Torrent'
        self.assertRaises(ValueError, create_magnet_link, hash, displayed_name, trackers)

    def test_magnet_link_with_empty_trackers(self):
        hash = '1234567890abcdef1234567890abcdef12345678'
        trackers = []
        displayed_name = 'My Torrent'
        self.assertRaises(ValueError, create_magnet_link, hash, displayed_name, trackers)

    def test_magnet_link_with_empty_displayed_name(self):
        hash = '1234567890abcdef1234567890abcdef12345678'
        trackers = ['http://tracker1.com', 'http://tracker2.com']
        displayed_name = ''
        self.assertRaises(ValueError, create_magnet_link, hash, displayed_name, trackers)

    def test_magnet_link_with_unicode_displayed_name(self):
        hash = '1234567890abcdef1234567890abcdef12345678'
        trackers = ['http://tracker1.com', 'http://tracker2.com']
        displayed_name = 'My Torrent 🤗'
        expected_output = 'magnet:?xt=urn:btih:1234567890abcdef1234567890abcdef12345678&dn=My%20Torrent%20%F0%9F%A4%97&tr=http%3A%2F%2Ftracker1.com&tr=http%3A%2F%2Ftracker2.com'
        self.assertEqual(create_magnet_link(hash, displayed_name, trackers), expected_output)

    def test_magnet_link_with_unicode_displayed_name_and_unicode_trackers(self):
        hash = '1234567890abcdef1234567890abcdef12345678'
        trackers = ['http://tracker1.com/🤗', 'http://tracker2.com/🤗']
        displayed_name = 'My Torrent 🤗'
        expected_output = 'magnet:?xt=urn:btih:1234567890abcdef1234567890abcdef12345678&dn=My%20Torrent%20%F0%9F%A4%97&tr=http%3A%2F%2Ftracker1.com%2F%F0%9F%A4%97&tr=http%3A%2F%2Ftracker2.com%2F%F0%9F%A4%97'
        self.assertEqual(create_magnet_link(hash, displayed_name, trackers), expected_output)


if __name__ == '__main__':
    unittest.main()



import unittest

from vha_toolbox import generate_table


class TableGeneratorTestCase(unittest.TestCase):
    def test_generate_table(self):
        rows = [['Alice', 28, 'USA'], ['Bob', 35, 'Canada']]
        header = ['Name', 'Age', 'Country']
        expected_html_table = '<table><tr><th>Name</th><th>Age</th><th>Country</th></tr><tr><td>Alice</td><td>28</td><td>USA</td></tr><tr><td>Bob</td><td>35</td><td>Canada</td></tr></table>'

        generated_html_table = generate_table(rows, header)
        self.assertEqual(generated_html_table, expected_html_table)

    def test_generate_table_no_header(self):
        rows = [['Alice', 28, 'USA'], ['Bob', 35, 'Canada']]
        expected_html_table = '<table><tr><td>Alice</td><td>28</td><td>USA</td></tr><tr><td>Bob</td><td>35</td><td>Canada</td></tr></table>'

        generated_html_table = generate_table(rows)
        self.assertEqual(generated_html_table, expected_html_table)

    def test_generate_table_no_rows(self):
        rows = []
        header = ['Name', 'Age', 'Country']
        expected_html_table = '<table><tr><th>Name</th><th>Age</th><th>Country</th></tr></table>'

        generated_html_table = generate_table(rows, header)
        self.assertEqual(generated_html_table, expected_html_table)

    def test_generate_table_no_rows_no_header(self):
        rows = []
        expected_html_table = '<table></table>'

        generated_html_table = generate_table(rows)
        self.assertEqual(generated_html_table, expected_html_table)

    def test_generate_table_with_incomplete_rows(self):
        rows = [['Alice', 28], ['Bob', 35, 'Canada']]
        header = ['Name', 'Age', 'Country']
        expected_html_table = '<table><tr><th>Name</th><th>Age</th><th>Country</th></tr><tr><td>Alice</td><td>28</td></tr><tr><td>Bob</td><td>35</td><td>Canada</td></tr></table>'

        generated_html_table = generate_table(rows, header)
        self.assertEqual(generated_html_table, expected_html_table)

    def test_generate_table_with_incomplete_rows_no_header(self):
        rows = [['Alice', 28], ['Bob', 35, 'Canada']]
        expected_html_table = '<table><tr><td>Alice</td><td>28</td></tr><tr><td>Bob</td><td>35</td><td>Canada</td></tr></table>'

        generated_html_table = generate_table(rows)
        self.assertEqual(generated_html_table, expected_html_table)

    def test_generate_table_with_pretty(self):
        rows = [['Alice', 28, 'USA'], ['Bob', 35, 'Canada']]
        header = ['Name', 'Age', 'Country']
        expected_html_table = '<table>\n    <tr>\n        <th>Name</th>\n        <th>Age</th>\n        <th>Country</th>\n    </tr>\n    <tr>\n        <td>Alice</td>\n        <td>28</td>\n        <td>USA</td>\n    </tr>\n    <tr>\n        <td>Bob</td>\n        <td>35</td>\n        <td>Canada</td>\n    </tr>\n</table>'

        generated_html_table = generate_table(rows, header, pretty=True)
        self.assertEqual(generated_html_table, expected_html_table)

    def test_generate_table_with_pretty_no_header(self):
        rows = [['Alice', 28, 'USA'], ['Bob', 35, 'Canada']]
        expected_html_table = '<table>\n    <tr>\n        <td>Alice</td>\n        <td>28</td>\n        <td>USA</td>\n    </tr>\n    <tr>\n        <td>Bob</td>\n        <td>35</td>\n        <td>Canada</td>\n    </tr>\n</table>'

        generated_html_table = generate_table(rows, pretty=True)
        self.assertEqual(generated_html_table, expected_html_table)

    def test_generate_table_with_pretty_no_rows(self):
        rows = []
        header = ['Name', 'Age', 'Country']
        expected_html_table = '<table>\n    <tr>\n        <th>Name</th>\n        <th>Age</th>\n        <th>Country</th>\n    </tr>\n</table>'

        generated_html_table = generate_table(rows, header, pretty=True)
        self.assertEqual(generated_html_table, expected_html_table)

    def test_generate_table_with_pretty_no_rows_no_header(self):
        rows = []
        expected_html_table = '<table>\n</table>'

        generated_html_table = generate_table(rows, pretty=True)
        self.assertEqual(generated_html_table, expected_html_table)


if __name__ == '__main__':
    unittest.main()
